# Generated by Django 2.1.9 on 2019-06-19 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("courses", "0003_auto_20190612_1743")]

    operations = [
        migrations.AlterField(
            model_name="person",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=200, null=True, verbose_name="First name"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="last_name",
            field=models.CharField(
                blank=True, max_length=200, null=True, verbose_name="Last name"
            ),
        ),
    ]



"""
Large banner CMS plugin
"""
from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from richie.apps.core.defaults import PLUGINS_GROUP

from .defaults import LARGEBANNER_TEMPLATES
from .forms import LargeBannerForm
from .models import LargeBanner


@plugin_pool.register_plugin
class LargeBannerPlugin(CMSPluginBase):
    """
    CMSPlugin to customize a home page header.
    """

    cache = True
    form = LargeBannerForm
    model = LargeBanner
    module = PLUGINS_GROUP
    name = _("Large Banner")
    # Required from CMSPluginBase signature but not used since we override it
    # from render()
    render_template = LARGEBANNER_TEMPLATES[0][0]

    fieldsets = (
        (None, {"fields": ["title", "template"]}),
        (None, {"fields": ["content"]}),
        (_("Medias"), {"fields": ["background_image", "logo", "logo_alt_text"]}),
    )

    def render(self, context, instance, placeholder):
        context.update({"instance": instance, "placeholder": placeholder})
        self.render_template = instance.template
        return context



"""
Validate and clean request parameters for our endpoints using Django forms
"""
from functools import reduce

from django import forms
from django.conf import settings
from django.utils.translation import get_language

import arrow

from richie.apps.courses.models import CourseState

from .defaults import QUERY_ANALYZERS, RELATED_CONTENT_BOOST
from .filter_definitions import FILTERS, AvailabilityFilterDefinition

# Instantiate filter fields for each filter defined in settings
# It is of the form:
#     {
#         subjects: (
#             ArrayField(required=False, base_type=forms.CharField(max_length=50)),
#             True,
#         ),
#         subjects_aggs: (
#             ArrayField(required=False, base_type=forms.CharField(max_length=50)),
#             True,
#         ),
#         subjects_children_aggs: (
#             forms.CharField(required=False, max_length=50),
#             False,
#         ),
#         languages: (
#             MultipleChoiceField(required=False, choices=LANGUAGES),
#             True,
#         ),
#     }
# Where the first item is an instance of the form field and the second is a boolean
# indicating whether this formfield expects a list or a single value.
FILTER_FIELDS = {
    key: value
    for filter_definition in FILTERS.values()
    for key, value in filter_definition.get_form_fields().items()
}


class SearchForm(forms.Form):
    """Validate the query string params in a search request."""

    OBJECTS, FILTERS = "objects", "filters"

    SCOPE_CHOICES = ((OBJECTS, "Objects"), (FILTERS, "Filters"))

    limit = forms.IntegerField(required=False, min_value=1, initial=10)
    query = forms.CharField(required=False, min_length=3, max_length=100)
    offset = forms.IntegerField(required=False, min_value=0, initial=0)
    scope = forms.ChoiceField(required=False, choices=SCOPE_CHOICES)


class CourseSearchForm(SearchForm):
    """
    Validate the query string params in the course search request, connect them to filter
    definitions and generate Elasticsearch queries.
    """

    def __init__(self, *args, data=None, **kwargs):
        """
        Adapt the search form to handle filters:
        - Fix the QueryDict value getter to properly handle multi-value parameters,
        - Add a field instance to the form for each filter,
        - Define the `states` property as it is used by several methods.
        """
        # QueryDict/MultiValueDict breaks lists: we need to fix it
        data_fixed = (
            {
                k: data.getlist(k)
                # Form fields are marked to expect lists as input or not as explained above
                if (k in FILTER_FIELDS and FILTER_FIELDS[k][1] is True) else v[0]
                for k, v in data.lists()
            }
            if data
            else {}
        )

        super().__init__(data=data_fixed, *args, **kwargs)
        self.fields.update({k: v[0] for k, v in FILTER_FIELDS.items()})
        self.states = None

    def clean_availability(self):
        """
        Calculate and set the list of states relevant with the current availability filter.
        e.g. if we filter on OPEN courses, only the course runs in state 0 (ongoing open) or
        1 (future open) should be considered for sorting and computation of the course's state.
        """
        availabilities = self.cleaned_data.get("availability", [])
        if AvailabilityFilterDefinition.OPEN in availabilities:
            self.states = [
                CourseState.ONGOING_OPEN,
                CourseState.FUTURE_OPEN,
                CourseState.ARCHIVED_OPEN,
            ]
        elif AvailabilityFilterDefinition.ONGOING in availabilities:
            self.states = [CourseState.ONGOING_OPEN, CourseState.ONGOING_CLOSED]
        elif AvailabilityFilterDefinition.COMING_SOON in availabilities:
            self.states = [
                CourseState.FUTURE_OPEN,
                CourseState.FUTURE_NOT_YET_OPEN,
                CourseState.FUTURE_CLOSED,
            ]
        elif AvailabilityFilterDefinition.ARCHIVED in availabilities:
            self.states = [CourseState.ARCHIVED_OPEN, CourseState.ARCHIVED_CLOSED]

        return availabilities

    def get_script_fields(self):
        """
        Build the part of the Elasticseach query that defines script fields ie fields that can not
        be indexed because they are dynamic and shoud be calculated at query time:
        - languages and states: depend on the filters applied by the user so that we only take into
          consideration course runs that are interesting for this search.
        - ms_since_epoch: evolves with time to stay relevant on course dates even if the ES
          instance and/or the Django server are long running.

        Note: we use script storage to save time on the script compilation, which is an expensive
        operation. We'll only do it once at bootstrap time.
        """
        return {
            "state": {
                "script": {
                    "id": "state_field",
                    "params": {
                        "languages": self.cleaned_data.get("languages") or None,
                        "ms_since_epoch": arrow.utcnow().int_timestamp * 1000,
                        "states": self.states,
                    },
                }
            }
        }

    def get_queries(self):
        """
        Aggregate queries from each filter definition.
        The full text query is treated directly as it does not require heavy lifting.

        Returns:
        --------
            List[Dict]: a list of dictionaries each mapping the name of a filter with a query
                fragment that composes the global Elasticsearch query. For example:
                [
                    {
                        "key": "new",
                        "fragment": [{'term': {'is_new': True}}]
                    }
                    ...
                ]
        """
        queries = []

        # Add the query fragments of each filter definition to the list of queries
        for filter_definition in FILTERS.values():
            queries = queries + filter_definition.get_query_fragment(self.cleaned_data)

        # Full text search is a regular (multilingual) match query
        full_text = self.cleaned_data.get("query")
        if full_text:
            related_content_boost = getattr(
                settings, "RICHIE_RELATED_CONTENT_BOOST", RELATED_CONTENT_BOOST
            )
            lang = get_language()
            queries.append(
                {
                    "key": "query",
                    "fragment": [
                        {
                            "bool": {
                                "should": [
                                    {
                                        "multi_match": {
                                            "analyzer": QUERY_ANALYZERS[lang],
                                            "fields": [
                                                "description.*",
                                                "introduction.*",
                                                "title.*^50",
                                                f"categories_names.*^{related_content_boost}",
                                                f"organizations_names.*^{related_content_boost}",
                                                f"persons_names.*^{related_content_boost}",
                                            ],
                                            "query": full_text,
                                            "type": "best_fields",
                                        }
                                    },
                                    # Boost exact code matches so that they jump our state buckets
                                    # A closed course with a perfect code match should be
                                    # displayed before an open course with no perfect code match
                                    {
                                        "match": {
                                            "code": {
                                                "query": full_text,
                                                "analyzer": "code",
                                                "boost": 3000,
                                            }
                                        }
                                    },
                                    {
                                        "match": {
                                            "code": {
                                                "query": full_text,
                                                "analyzer": "code",
                                                "fuzziness": "AUTO",
                                            }
                                        }
                                    },
                                ]
                            }
                        }
                    ],
                }
            )

        return queries

    def build_es_query(self):
        """
        Build the actual Elasticsearch search query and aggregation query from the fragments
        returned by each filter definition.

        Returns:
        --------
            Tuple:
            - limit (int): the maximum number of results to be returned by Elasticsearch,
            - offset (int): the offset from which results are returned (for pagination),
            - query (Dict): the raw Elasticsearch query as per:
              https://elastic.co/guide/en/elasticsearch/reference/current/search.html
            - aggs (Dict): the raw Elasticsearch aggregation as per:
              https://elastic.co/guide/en/elasticsearch/reference/current/search-aggregations.html

        """
        # queries is an array of individual queries that will be combined through "bool" before
        # we pass them to ES. See the docs on bool queries.
        # https://www.elastic.co/guide/en/elasticsearch/reference/current/query-dsl-bool-query.html
        queries = self.get_queries()

        # Concatenate all the sub-queries lists together to form the queries list
        query = {
            "function_score": {
                "query": {
                    "bool": {
                        # Always filter out courses that are not flagged for listing
                        "filter": {"term": {"is_listed": True}},
                        "must":
                        # queries => map(pluck("fragment")) => flatten()
                        [
                            clause
                            for kf_pair in queries
                            for clause in kf_pair["fragment"]
                        ],
                    }
                },
                "boost_mode": "replace",
                "script_score": {
                    "script": {
                        "id": "score",
                        "params": {
                            "languages": self.cleaned_data.get("languages") or None,
                            "ms_since_epoch": arrow.utcnow().timestamp() * 1000,
                            "states": self.states,
                        },
                    }
                },
            }
        }

        # Concatenate our hardcoded filters query fragments with organizations and categories
        # terms aggregations build on-the-fly
        aggs = {
            "all_courses": {
                "global": {},
                "aggregations": reduce(
                    # Merge all the partial aggregations dicts together
                    lambda acc, aggs_fragment: {**acc, **aggs_fragment},
                    # Generate a partial aggregations dict (an aggs_fragment) for each filter
                    [
                        filter.get_aggs_fragment(queries, self.cleaned_data)
                        for filter in FILTERS.values()
                    ],
                    {},
                ),
            }
        }

        return (
            self.cleaned_data.get("limit"),
            self.cleaned_data.get("offset") or 0,
            query,
            aggs,
        )


class ItemSearchForm(SearchForm):
    """Generate Elasticsearch queries for the category/organization indices."""

    def __init__(self, *args, data=None, **kwargs):
        """Fix the QueryDict value getter to properly handle multi-value parameters."""
        # QueryDict/MultiValueDict breaks lists: we need to fix it
        data_fixed = {k: v[0] for k, v in data.lists()} if data else {}
        super().__init__(data=data_fixed, *args, **kwargs)

    def build_es_query(self, kind=None):
        """
        Build the actual Elasticsearch search query for category/organization indices.

        Returns:
        --------
            Tuple:
            - limit (int): the maximum number of results to be returned by Elasticsearch,
            - offset (int): the offset from which results are returned (for pagination),
            - query (Dict): the raw Elasticsearch query as per:
              https://elastic.co/guide/en/elasticsearch/reference/current/search.html

        """
        # Create an array to put the clauses of our bool/must query
        clauses = []

        # Add a term filter by kind for the objects that need it (eg. `kind="subjects"`)
        if kind:
            clauses.append({"term": {"kind": kind}})

        # Add a match on the name field if it was handed by the client
        full_text = self.cleaned_data.get("query")
        if full_text:
            clauses.append(
                {
                    "multi_match": {
                        "analyzer": QUERY_ANALYZERS[get_language()],
                        "fields": ["title.*"],
                        "query": full_text,
                    }
                }
            )

        # Build the query around the clauses if there are any
        if clauses:
            query = {"query": {"bool": {"must": clauses}}}
        # Build a match_all query by default
        else:
            query = {"query": {"match_all": {}}}

        return (
            self.cleaned_data.get("limit"),
            self.cleaned_data.get("offset") or 0,
            query,
        )



"""
LTI consumer plugin models
"""
from urllib.parse import unquote

from django.conf import settings
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.functional import lazy
from django.utils.translation import gettext_lazy as _

import exrex
from cms.models.pluginmodel import CMSPlugin
from cms.utils import get_current_site
from oauthlib import oauth1

from .defaults import INSTRUCTOR, PREDEFINED_LTI_PROVIDERS, STUDENT


class LTIConsumer(CMSPlugin):
    """
    LTI consumer plugin model.
    """

    RESOURCE_NAME = "lti-consumer"

    url = models.URLField(
        verbose_name=_("LTI url"),
        help_text=_(
            "For a predefined provider, leave this field empty for uploading new content."
        ),
        null=False,
        blank=True,
    )
    lti_provider_id = models.CharField(
        verbose_name=_("Predefined LTI provider"),
        max_length=50,
        null=True,
        blank=True,
        choices=lazy(lambda: PREDEFINED_LTI_PROVIDERS, tuple)(),
        help_text=_("Please choose a predefined provider or fill fields below."),
    )
    oauth_consumer_key = models.CharField(max_length=50, null=True, blank=True)
    shared_secret = models.CharField(max_length=50, null=True, blank=True)
    is_automatic_resizing = models.BooleanField(default=True, blank=True)
    inline_ratio = models.FloatField(
        default=0.5625,  # default is 16/9
        blank=True,
        validators=[MinValueValidator(0.1), MaxValueValidator(10)],
    )

    # pylint: disable=signature-differs
    def save(self, *args, **kwargs):
        """
        Initialize the url attribute if a predefined provider is used.
        """
        lti_provider = self.lti_provider

        if lti_provider and not self.url:
            is_regex = lti_provider.get("is_base_url_regex")
            base_url = lti_provider.get("base_url", "")
            if is_regex:
                self.url = str(exrex.getone(base_url))
            else:
                self.url = base_url

        super().save(*args, **kwargs)

    @property
    def lti_provider(self):
        """
        Returns current predefined LTI provider settings
        """
        return getattr(settings, "RICHIE_LTI_PROVIDERS", {}).get(
            self.lti_provider_id, {}
        )

    def get_inline_ratio_percentage(self):
        """
        Returns actual value to use for `inline_ratio` setting as a percentage
        """
        return 100 * (
            self.lti_provider.get("inline_ratio", 0.5625)
            if self.lti_provider_id
            else self.inline_ratio
        )

    def get_resource_link_id(self):
        """Use the plugin id as resource_link_id field."""
        return str(self.id)

    def get_content_parameters(self, edit=False):
        """
        Convenient wrapper to authorize and return required parameters
        for LTI content consumption
        """
        role = INSTRUCTOR if edit else STUDENT
        site = get_current_site()

        lti_parameters = {
            "lti_message_type": "basic-lti-launch-request",
            "lti_version": "LTI-1p0",
            "resource_link_id": self.get_resource_link_id(),
            "context_id": site.domain,
            "user_id": "richie",
            "lis_person_contact_email_primary": "",
            "roles": role,
        }

        # Get credentials from the predefined LTI provider if any, or from the model otherwise
        oauth_consumer_key = (
            self.lti_provider.get("oauth_consumer_key", "")
            if self.lti_provider_id
            else self.oauth_consumer_key
        )
        shared_secret = (
            self.lti_provider.get("shared_secret", "")
            if self.lti_provider_id
            else self.shared_secret
        )
        client = oauth1.Client(
            client_key=oauth_consumer_key, client_secret=shared_secret
        )
        _uri, headers, _body = client.sign(
            self.url,
            http_method="POST",
            body=lti_parameters,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )

        oauth_dict = dict(
            param.strip().replace('"', "").split("=")
            for param in headers["Authorization"].split(",")
        )
        oauth_dict["oauth_signature"] = unquote(oauth_dict["oauth_signature"])
        oauth_dict["oauth_nonce"] = oauth_dict.pop("OAuth oauth_nonce")

        oauth_dict.update(lti_parameters)
        return oauth_dict

    def __str__(self):
        return self.url



"""middlewares used with python social auth."""
from django.conf import settings

from cms.models import Page
from cms.utils import get_language_from_request

from social_django.middleware import (  # isort:skip
    SocialAuthExceptionMiddleware as BaseSocialAuthExceptionMiddleware,
)

SOCIAL_ERROR_REVERSE_ID = getattr(settings, "SOCIAL_ERROR_REVERSE_ID", None)


class SocialAuthExceptionMiddleware(BaseSocialAuthExceptionMiddleware):
    """Middleware extending social-auth middleware, overriding get_redirect_uri."""

    def get_redirect_uri(self, request, exception):
        """
        Check if an error page exists and returns its url. Fallback on parent function otherwise.
        """
        try:
            page = Page.objects.get(reverse_id=SOCIAL_ERROR_REVERSE_ID)
        except Page.DoesNotExist:
            return super.get_redirect_uri(request, exception)
        else:
            return page.get_absolute_url(language=get_language_from_request(request))



"""Utils for the demo app of the Richie project."""
import os
import random

from django.conf import settings


def pick_image(image_type):
    """
    This function can be passed to our factories's `fill_*` method to dynamically pick a random
    image of a given type.

    It randomly picks-up an image from targeted directory (core fixtures directory +
    image type dir).

    Arguments:
        image_type (string): Directory name in fixtures directory that contains some images to
            pick up.

    Returns:
        callable: A function that picks a random image path for a given type of image (logo,
        banner,...)
    """

    def _pick_random(filename=None):
        """
        Pick a random file path from fixtures within the image type passed as argument to the
        parent function.
        """
        default_directory = os.path.join(os.path.dirname(__file__), "fixtures")
        fixtures_directory = getattr(
            settings, "RICHIE_DEMO_FIXTURES_DIR", default_directory
        )
        image_directory = os.path.join(fixtures_directory, image_type)

        if not os.path.exists(image_directory):
            # Fallback to the fixtures directory in the demo app
            image_directory = os.path.join(default_directory, image_type)

        filename = filename or random.choice(os.listdir(image_directory))  # nosec
        return os.path.join(image_directory, filename)

    return _pick_random



"""
Courses factories
"""
import random
from collections import namedtuple
from datetime import datetime, timedelta

from django.utils import timezone
from django.utils.translation import gettext_lazy as _

import factory
import pytz
from cms.api import add_plugin

from richie.plugins.nesteditem.defaults import ACCORDION

from ..core.defaults import ALL_LANGUAGES
from ..core.factories import (
    FilerImageFactory,
    PageExtensionDjangoModelFactory,
    PageFactory,
    create_text_plugin,
    image_getter,
)
from . import defaults, models
from .defaults import ROLE_CHOICES

VideoSample = namedtuple("VideoSample", ["label", "image", "url"])

VIDEO_SAMPLE_LINKS = (
    VideoSample(
        "Anant Agarwal: Why massively open online courses (still) matter",
        "cover1.jpg",
        "//www.youtube.com/embed/rYwTA5RA9eU",
    ),
    VideoSample(
        "Installing Open edX", "cover2.jpg", "//www.youtube.com/embed/YDm6bAPxeg0"
    ),
    VideoSample(
        "Open edX Conference 2018 Opening and Welcome remarks",
        "cover3.jpg",
        "//www.youtube.com/embed/zzx6MgBAbCc",
    ),
)


def associate_extensions(extension, related_extensions, slot, plugin_type):
    """Add plugins to related pages that point to a given page."""
    for related_extension in related_extensions:
        placeholder = extension.extended_object.placeholders.get(slot=slot)
        for language in extension.extended_object.get_languages():
            add_plugin(
                language=language,
                placeholder=placeholder,
                plugin_type=plugin_type,
                **{"page": related_extension.extended_object},
            )


class BLDPageExtensionDjangoModelFactory(PageExtensionDjangoModelFactory):
    """
    This mixin mutualizes filling placeholders for banner, logo and description fields because
    several models share these 3 placeholders.
    """

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_banner(self, create, extracted, **kwargs):
        """
        Add a banner with a random image
        """
        if create and extracted:
            banner_placeholder = self.extended_object.placeholders.get(slot="banner")

            if isinstance(extracted, str):
                banner = image_getter(extracted)
            elif callable(extracted):
                banner = image_getter(extracted())
            else:
                banner = FilerImageFactory(original_filename="banner.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=banner_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=banner,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_logo(self, create, extracted, **kwargs):
        """
        Add a logo with a random image
        """
        if create and extracted:
            logo_placeholder = self.extended_object.placeholders.get(slot="logo")

            if isinstance(extracted, str):
                logo = image_getter(extracted)
            elif callable(extracted):
                logo = image_getter(extracted())
            elif isinstance(extracted, dict):
                logo = FilerImageFactory(**extracted)
            else:
                logo = FilerImageFactory(original_filename="logo.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=logo_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=logo,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_description(self, create, extracted, **kwargs):
        """
        Add a text plugin for description with a long random text
        """
        if create and extracted:
            create_text_plugin(
                self.extended_object,
                "description",
                nb_paragraphs=random.randint(2, 4),  # nosec
                languages=self.extended_object.get_languages(),
            )


class OrganizationFactory(BLDPageExtensionDjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful organization page extensions
    in our tests.
    """

    class Meta:
        model = models.Organization
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_reverse_id",
            "page_template",
            "page_title",
        ]

    # fields concerning the related page
    page_template = models.Organization.PAGE["template"]

    @factory.lazy_attribute_sequence
    def code(self, sequence):
        """
        Since `name` is required, let's just slugify it to get a meaningful code (and keep it
        below 100 characters)
        """
        short_slug = self.extended_object.get_slug()[:90]
        return f"{short_slug:s}-{sequence:d}"

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_categories(self, create, extracted, **kwargs):
        """Add categories plugin to organization from a given list of category instances."""
        if create and extracted:
            associate_extensions(self, extracted, "categories", "CategoryPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def with_permissions(self, create, extracted, **kwargs):
        """
        Create a page role with related group, page permission and filer folder.
        """
        if create and extracted:
            self.create_page_role()


class PageRoleFactory(factory.django.DjangoModelFactory):
    """A factory to automatically generate random yet meaningful page roles."""

    class Meta:
        model = models.PageRole

    page = factory.SubFactory(PageFactory)
    role = factory.Iterator([d[0] for d in ROLE_CHOICES])


class CourseFactory(PageExtensionDjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful course page extensions
    and their related page in our tests.
    """

    class Meta:
        model = models.Course
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_template",
            "page_title",
        ]

    # fields concerning the related page
    page_template = models.Course.PAGE["template"]

    code = factory.Sequence(lambda k: f"{k:05d}")

    # pylint: disable=no-self-use
    @factory.lazy_attribute
    def duration(self):
        """Generate a random duration for the course between 1 day and 10 months."""
        return [
            random.randint(1, 10),  # nosec
            random.choice(list(defaults.TIME_UNITS.keys())[2:]),  # nosec
        ]

    # pylint: disable=no-self-use
    @factory.lazy_attribute
    def effort(self):
        """Generate a random effort for the course between 1 minute and 500 hours."""
        return [
            random.randint(1, 500),  # nosec
            random.choice(list(defaults.EFFORT_UNITS.keys())),  # nosec
        ]

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_teaser(self, create, extracted, **kwargs):
        """
        Add a video plugin for teaser with a random url
        """

        if create and extracted:
            for language in self.extended_object.get_languages():

                placeholder = self.extended_object.placeholders.get(
                    slot="course_teaser"
                )

                video_sample = (
                    extracted
                    if isinstance(extracted, VideoSample)
                    else random.choice(VIDEO_SAMPLE_LINKS)  # nosec
                )

                add_plugin(
                    language=language,
                    placeholder=placeholder,
                    plugin_type="VideoPlayerPlugin",
                    label=video_sample.label,
                    embed_link=video_sample.url,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_cover(self, create, extracted, **kwargs):
        """
        Add a picture plugin for course cover with a random image
        """
        if create and extracted:
            cover_placeholder = self.extended_object.placeholders.get(
                slot="course_cover"
            )

            if isinstance(extracted, str):
                cover = image_getter(extracted)
            elif callable(extracted):
                cover = image_getter(extracted())
            elif isinstance(extracted, dict):
                cover = FilerImageFactory(**extracted)
            else:
                cover = FilerImageFactory(original_filename="cover.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=cover_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=cover,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_team(self, create, extracted, **kwargs):
        """
        Add person plugin for course team from given person instance list
        """
        if create and extracted:
            associate_extensions(self, extracted, "course_team", "PersonPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_categories(self, create, extracted, **kwargs):
        """
        Add categories plugin to course from a given list of category instances.
        """

        if create and extracted:
            associate_extensions(self, extracted, "course_categories", "CategoryPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_icons(self, create, extracted, **kwargs):
        """Add category plugin to course from a given list of category instances."""

        if create and extracted:
            associate_extensions(self, extracted, "course_icons", "CategoryPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_organizations(self, create, extracted, **kwargs):
        """
        Add organizations plugin to course from a given list of organization instances.
        """

        if create and extracted:
            associate_extensions(
                self, extracted, "course_organizations", "OrganizationPlugin"
            )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_licences(self, create, extracted, **kwargs):
        """
        Add licence plugin for course licence placeholders from given licence
        instance list
        """
        if create and extracted:
            for language in self.extended_object.get_languages():
                for slot, licence in extracted:
                    placeholder = self.extended_object.placeholders.get(slot=slot)
                    add_plugin(
                        language=language,
                        placeholder=placeholder,
                        plugin_type="LicencePlugin",
                        **{"licence": licence},
                    )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_plan(self, create, extracted, **kwargs):
        """Add a course plan composed of random nested items."""
        if create and extracted:
            for language in self.extended_object.get_languages():
                placeholder = self.extended_object.placeholders.get(slot="course_plan")
                container = add_plugin(
                    language=language,
                    placeholder=placeholder,
                    plugin_type="NestedItemPlugin",
                    variant=ACCORDION,
                )
                for chapter in range(random.randint(2, 4)):  # nosec
                    chapter_plugin = add_plugin(
                        language=language,
                        placeholder=placeholder,
                        plugin_type="NestedItemPlugin",
                        target=container,
                        content=f"Chapter {chapter:d}",
                        variant=ACCORDION,
                    )
                    for part in range(random.randint(2, 4)):  # nosec
                        add_plugin(
                            language=language,
                            placeholder=placeholder,
                            plugin_type="NestedItemPlugin",
                            target=chapter_plugin,
                            content=f"Part {part:d}",
                            variant=ACCORDION,
                        )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_texts(self, create, extracted, **kwargs):
        """
        A shortand to fill some placeholder content with a text plugin.

        Placeholder slot names to fill are given from ``extracted`` argument
        in a list of slot names.
        """
        if create and extracted:
            for slot, plugin_type in extracted.items():
                create_text_plugin(
                    self.extended_object,
                    slot,
                    nb_paragraphs=1,
                    languages=self.extended_object.get_languages(),
                    is_html=plugin_type != "PlainTextPlugin",
                    plugin_type=plugin_type,
                )


class CourseRunFactory(factory.django.DjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful course runs in our tests.

    Random dates are proposed realistically so that:
        - now <= start
        - enrollment_start <= start <= end
        - enrollment_start <= enrollment_end <= end
    """

    class Meta:
        model = models.CourseRun

    direct_course = factory.SubFactory(CourseFactory)
    title = factory.Sequence(_("Run {:d}").format)
    resource_link = factory.Faker("uri")
    sync_mode = models.CourseRunSyncMode.SYNC_TO_PUBLIC

    # pylint: disable=no-self-use
    @factory.lazy_attribute
    def languages(self):
        """
        Compute a random set of languages from the complete list of Django supported languages.
        """
        return [
            language[0]
            for language in random.sample(ALL_LANGUAGES, random.randint(1, 5))  # nosec
        ]

    # pylint: disable=no-self-use
    @factory.lazy_attribute
    def start(self):
        """
        A start datetime for the course run is chosen randomly in the future (it can
        of course be forced if we want something else), then the other significant dates
        for the course run are chosen randomly in periods that make sense with this start date.
        """
        now = timezone.now()
        period = timedelta(days=200)
        return datetime.utcfromtimestamp(
            random.randrange(  # nosec
                int((now - period).timestamp()), int((now + period).timestamp())
            )
        ).replace(tzinfo=pytz.utc)

    @factory.lazy_attribute
    def end(self):
        """
        The end datetime is at a random duration after the start datetme (we pick within 90 days).
        """
        if not self.start:
            return None
        period = timedelta(days=90)
        return datetime.utcfromtimestamp(
            random.randrange(  # nosec
                int(self.start.timestamp()), int((self.start + period).timestamp())
            )
        ).replace(tzinfo=pytz.utc)

    @factory.lazy_attribute
    def enrollment_start(self):
        """
        The start of enrollment is a random datetime before the start datetime.
        """
        if not self.start:
            return None
        period = timedelta(days=90)
        return datetime.utcfromtimestamp(
            random.randrange(  # nosec
                int((self.start - period).timestamp()), int(self.start.timestamp())
            )
        ).replace(tzinfo=pytz.utc)

    @factory.lazy_attribute
    def enrollment_end(self):
        """
        The end of enrollment is a random datetime between the start of enrollment
        and the end of the course.
        If the enrollment start and end datetimes have been forced to incoherent dates,
        then just don't set any end of enrollment...
        """
        if not self.start:
            return None
        enrollment_start = self.enrollment_start or self.start - timedelta(
            days=random.randint(1, 90)  # nosec
        )
        max_enrollment_end = self.end or self.start + timedelta(
            days=random.randint(1, 90)  # nosec
        )
        max_enrollment_end = max(
            enrollment_start + timedelta(hours=1), max_enrollment_end
        )
        return datetime.utcfromtimestamp(
            random.randrange(  # nosec
                int(enrollment_start.timestamp()), int(max_enrollment_end.timestamp())
            )
        ).replace(tzinfo=pytz.utc)

    @factory.lazy_attribute
    def enrollment_count(self):
        """
        The number of enrollments of a course run is a random integer between 0 and 10,000.
        """
        return random.randint(0, 10000)  # nosec


class CategoryFactory(BLDPageExtensionDjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful category page extensions
    and their related page in our tests.
    """

    class Meta:
        model = models.Category
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_reverse_id",
            "page_template",
            "page_title",
        ]

    # fields concerning the related page
    page_template = models.Category.PAGE["template"]

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_icon(self, create, extracted, **kwargs):
        """Add an icon with a random image."""
        if create and extracted:
            icon_placeholder = self.extended_object.placeholders.get(slot="icon")

            if isinstance(extracted, str):
                icon = image_getter(extracted)
            elif callable(extracted):
                icon = image_getter(extracted())
            elif isinstance(extracted, dict):
                icon = FilerImageFactory(**extracted)
            else:
                icon = FilerImageFactory(original_filename="icon.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=icon_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=icon,
                )


class LicenceLogoImageFactory(FilerImageFactory):
    """Image field factory for Licence."""

    file = factory.django.ImageField()


class LicenceFactory(factory.django.DjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful licences.
    """

    class Meta:
        model = models.Licence

    name = factory.Faker("sentence", nb_words=3)
    logo = factory.SubFactory(LicenceLogoImageFactory)
    url = factory.Faker("uri")
    content = factory.Faker("text", max_nb_chars=300)


class BlogPostFactory(PageExtensionDjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful blogpost extensions
    in our tests.
    """

    class Meta:
        model = models.BlogPost
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_template",
            "page_title",
        ]

    # fields concerning the related page
    page_template = models.BlogPost.PAGE["template"]

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_author(self, create, extracted, **kwargs):
        """
        Add person plugin for blog post author from given person instance list
        """

        if create and extracted:
            for language in self.extended_object.get_languages():
                placeholder = self.extended_object.placeholders.get(slot="author")

                for person in extracted:
                    add_plugin(
                        language=language,
                        placeholder=placeholder,
                        plugin_type="PersonPlugin",
                        **{"page": person.extended_object},
                    )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_body(self, create, extracted, **kwargs):
        """
        Add a text plugin for body with a long random text
        """
        if create and extracted:
            create_text_plugin(
                self.extended_object,
                "body",
                nb_paragraphs=random.randint(4, 6),  # nosec
                languages=self.extended_object.get_languages(),
                plugin_type="TextPlugin",
            )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_categories(self, create, extracted, **kwargs):
        """
        Add categories plugin to blog post from a given list of category
        instances.
        """

        if create and extracted:
            associate_extensions(self, extracted, "categories", "CategoryPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_cover(self, create, extracted, **kwargs):
        """
        Add a picture plugin for blog post cover with a random image
        """
        if create and extracted:
            cover_placeholder = self.extended_object.placeholders.get(slot="cover")

            if isinstance(extracted, str):
                cover = image_getter(extracted)
            elif callable(extracted):
                cover = image_getter(extracted())
            elif isinstance(extracted, dict):
                cover = FilerImageFactory(**extracted)
            else:
                cover = FilerImageFactory(original_filename="cover.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=cover_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=cover,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_excerpt(self, create, extracted, **kwargs):
        """
        Add a plain text plugin for excerpt with a short random text
        """
        if create and extracted:
            placeholder = self.extended_object.placeholders.get(slot="excerpt")

            for language in self.extended_object.get_languages():
                text = factory.Faker(
                    "text", max_nb_chars=random.randint(50, 100)  # nosec
                ).evaluate(None, None, {"locale": language})
                add_plugin(
                    language=language,
                    placeholder=placeholder,
                    plugin_type="PlainTextPlugin",
                    body=text,
                )


class PersonFactory(PageExtensionDjangoModelFactory):
    """
    Person factory to generate random yet realistic person's name and title
    """

    first_name = factory.Faker("first_name")
    last_name = factory.Faker("last_name")
    person_title = factory.Faker("prefix")

    @factory.lazy_attribute
    def page_title(self):
        """
        Build the page title from the person's title and names
        """
        # Don't always set a title
        person_title = random.choice([None, self.person_title])  # nosec

        # Join the names that are not null into a string
        names = [person_title, self.first_name, self.last_name]
        return " ".join([n for n in names if n is not None])

    class Meta:
        model = models.Person
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_template",
            "page_title",
            "first_name",
            "last_name",
            "person_title",
        ]

    # fields concerning the related page
    page_template = models.Person.PAGE["template"]

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_categories(self, create, extracted, **kwargs):
        """Add categories plugin to person from a given list of category instances."""
        if create and extracted:
            associate_extensions(self, extracted, "categories", "CategoryPlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_portrait(self, create, extracted, **kwargs):
        """
        Add a portrait with a random image
        """
        if create and extracted:
            portrait_placeholder = self.extended_object.placeholders.get(
                slot="portrait"
            )

            if isinstance(extracted, str):
                portrait = image_getter(extracted)
            elif callable(extracted):
                portrait = image_getter(extracted())
            elif isinstance(extracted, dict):
                portrait = FilerImageFactory(**extracted)
            else:
                portrait = FilerImageFactory(original_filename="portrait.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=portrait_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=portrait,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_bio(self, create, extracted, **kwargs):
        """
        Add a text plugin for bio with a long random text
        """
        if create and extracted:
            create_text_plugin(
                self.extended_object,
                "bio",
                nb_paragraphs=1,
                languages=self.extended_object.get_languages(),
                is_html=False,
                plugin_type="PlainTextPlugin",
            )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_maincontent(self, create, extracted, **kwargs):
        """
        Add a text plugin for maincontent with a long random text
        """
        if create and extracted:
            create_text_plugin(
                self.extended_object,
                "maincontent",
                nb_paragraphs=1,
                languages=self.extended_object.get_languages(),
            )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_organizations(self, create, extracted, **kwargs):
        """
        Add organizations plugin to person from a given list of organization instances.
        """

        if create and extracted:
            associate_extensions(self, extracted, "organizations", "OrganizationPlugin")


class ProgramFactory(PageExtensionDjangoModelFactory):
    """
    A factory to automatically generate random yet meaningful program extensions
    in our tests.
    """

    class Meta:
        model = models.Program
        exclude = [
            "page_in_navigation",
            "page_languages",
            "page_parent",
            "page_template",
            "page_title",
        ]

    # fields concerning the related page
    page_template = models.Program.PAGE["template"]

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_body(self, create, extracted, **kwargs):
        """
        Add a text plugin for body with a long random text
        """
        if create and extracted:
            create_text_plugin(
                self.extended_object,
                "program_body",
                nb_paragraphs=random.randint(4, 6),  # nosec
                languages=self.extended_object.get_languages(),
                plugin_type="TextPlugin",
            )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_courses(self, create, extracted, **kwargs):
        """
        Add plugins for this program to each course in the given list of course
        instances.
        """
        if create and extracted:
            associate_extensions(self, extracted, "program_courses", "CoursePlugin")

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_cover(self, create, extracted, **kwargs):
        """
        Add a picture plugin for program cover with a random image
        """
        if create and extracted:
            cover_placeholder = self.extended_object.placeholders.get(
                slot="program_cover"
            )

            if isinstance(extracted, str):
                cover = image_getter(extracted)
            elif callable(extracted):
                cover = image_getter(extracted())
            elif isinstance(extracted, dict):
                cover = FilerImageFactory(**extracted)
            else:
                cover = FilerImageFactory(original_filename="cover.jpg")

            for language in self.extended_object.get_languages():
                add_plugin(
                    language=language,
                    placeholder=cover_placeholder,
                    plugin_type="SimplePicturePlugin",
                    picture=cover,
                )

    @factory.post_generation
    # pylint: disable=unused-argument
    def fill_excerpt(self, create, extracted, **kwargs):
        """
        Add a plain text plugin for excerpt with a short random text
        """
        if create and extracted:
            placeholder = self.extended_object.placeholders.get(slot="program_excerpt")

            for language in self.extended_object.get_languages():
                text = factory.Faker(
                    "text", max_nb_chars=random.randint(50, 100)  # nosec
                ).evaluate(None, None, {"locale": language})
                add_plugin(
                    language=language,
                    placeholder=placeholder,
                    plugin_type="PlainTextPlugin",
                    body=text,
                )



"""
Declare and configure the model for the person application
"""
from django.db import models
from django.utils.translation import gettext_lazy as _

from cms.api import Page
from cms.extensions.extension_pool import extension_pool
from cms.models.pluginmodel import CMSPlugin

from ...core.models import BasePageExtension, EsIdMixin
from ..defaults import PERSONS_PAGE


class Person(EsIdMixin, BasePageExtension):
    """
    The person page extension represents and records people information.
    It could be a course or news article author.

    This model should be used to record structured data about the person whereas the
    associated page object is where we record the less structured information to display on the
    page to present the person.
    """

    PAGE = PERSONS_PAGE

    class Meta:
        db_table = "richie_person"
        verbose_name = _("person")
        verbose_name_plural = _("persons")

    def __str__(self):
        """Human representation of a person."""
        model = self._meta.verbose_name.title()
        name = self.extended_object.get_title()
        return f"{model:s}: {name:s}"

    def save(self, *args, **kwargs):
        """
        Enforce validation on each instance save
        """
        self.full_clean()
        super().save(*args, **kwargs)

    def get_courses(self, language=None):
        """
        Return a query to get the courses related to this person ie for which a plugin for
        this person is linked to the course page via any placeholder.
        """
        return self.get_reverse_related_page_extensions(
            "course", language=language
        ).filter(extended_object__node__parent__cms_pages__course__isnull=True)

    def get_blogposts(self, language=None):
        """
        Return a query to get the blogposts this person wrote ie for which a
        plugin for this person is linked to the blogpost page via any placeholder.
        """
        return self.get_reverse_related_page_extensions("blogpost", language=language)


class PersonPluginModel(CMSPlugin):
    """
    Person plugin model handles the relation from PersonPlugin
    to their Person instance
    """

    page = models.ForeignKey(
        Page,
        on_delete=models.CASCADE,
        limit_choices_to={"publisher_is_draft": True, "person__isnull": False},
        related_name="person_plugins",
    )
    bio = models.TextField(
        _("Custom bio"),
        null=True,
        blank=True,
        default="",
        help_text=_(
            "Optional: provide a custom bio "
            "(if you leave it empty, it will display the person's bio)."
        ),
    )

    class Meta:
        db_table = "richie_person_plugin"
        verbose_name = _("person plugin")
        verbose_name_plural = _("person plugins")

    def __str__(self):
        """Human representation of a page plugin"""
        return self.page.get_title()


extension_pool.register(Person)



# Generated by Django 2.2.12 on 2020-05-11 20:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("nesteditem", "0002_auto_20200417_1237"),
    ]

    operations = [
        migrations.AlterField(
            model_name="nesteditem",
            name="variant",
            field=models.CharField(
                choices=[("list", "List"), ("accordion", "Accordion")],
                default="list",
                help_text="Form factor variant",
                max_length=50,
                verbose_name="Variant",
            ),
        ),
    ]



"""
Backend to connect Open edX richie with an LMS
"""
import logging
import re

import requests
from requests.auth import AuthBase

from ..serializers import SyncCourseRunSerializer
from .base import BaseLMSBackend

logger = logging.getLogger(__name__)


def split_course_key(key):
    """Split an OpenEdX course key by organization, course and course run codes.

    We first try splitting the key as a version 1 key (course-v1:org+course+run)
    and fallback the old version (org/course/run).
    """
    if key.startswith("course-v1:"):
        organization, course, run = key[10:].split("+")
    else:
        organization, course, run = key.split("/")

    return organization, course, run


class EdXTokenAuth(AuthBase):
    """Attach HTTP token authentication to the given Request object."""

    def __init__(self, token):
        """Set-up token value in the instance."""
        self.token = token

    def __call__(self, request):
        """Modify and return the request."""
        request.headers.update(
            {"X-Edx-Api-Key": self.token, "Content-Type": "application/json"}
        )
        return request


class TokenAPIClient(requests.Session):
    """
    A :class:`requests.Session` that automatically authenticates against edX's preferred
    authentication method up to Dogwood, given a secret token.

    For more usage details, see documentation of the :class:`requests.Session` object:
    https://requests.readthedocs.io/en/master/user/advanced/#session-objects
    """

    def __init__(self, token, *args, **kwargs):
        """Extending the session object by setting the authentication token."""
        super().__init__(*args, **kwargs)
        self.auth = EdXTokenAuth(token)


class EdXLMSBackend(BaseLMSBackend):
    """LMS backend for Richie tested with Open EdX Dogwood to Hawthorn."""

    @property
    def api_client(self):
        """Instantiate and return an edx token API client."""
        return TokenAPIClient(self.configuration["API_TOKEN"])

    def extract_course_id(self, url):
        """Extract the LMS course id from the course run url."""
        return re.match(self.configuration["COURSE_REGEX"], url).group("course_id")

    def extract_course_number(self, data):
        """Extract the LMS course number from data dictionary."""
        course_id = self.extract_course_id(data.get("resource_link"))
        return split_course_key(course_id)[1]

    def clean_course_run_data(self, data):
        """Remove course run's protected fields to the data dictionnary."""
        return {
            key: value
            for (key, value) in data.items()
            if key not in self.configuration.get("COURSE_RUN_SYNC_NO_UPDATE_FIELDS", [])
        }

    @staticmethod
    def get_course_run_serializer(data, partial=False):
        """Prepare data and return a bound serializer."""
        return SyncCourseRunSerializer(data=data, partial=partial)



# Generated by Django 3.1.12 on 2021-06-03 16:17

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("lti_consumer", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="lticonsumer",
            name="inline_ratio",
            field=models.FloatField(
                blank=True,
                default=0.5625,
                validators=[
                    django.core.validators.MinValueValidator(0.1),
                    django.core.validators.MaxValueValidator(10),
                ],
            ),
        ),
        migrations.AddField(
            model_name="lticonsumer",
            name="is_automatic_resizing",
            field=models.BooleanField(blank=True, default=True),
        ),
        migrations.AlterField(
            model_name="lticonsumer",
            name="lti_provider_id",
            field=models.CharField(
                blank=True,
                choices=[
                    (None, "Custom provider configuration"),
                    ("lti_provider_test", "LTI Provider Test Video"),
                ],
                help_text="Please choose a predefined provider or fill fields below.",
                max_length=50,
                null=True,
                verbose_name="Predefined LTI provider",
            ),
        ),
    ]



# Generated by Django 2.1.8 on 2019-05-14 19:20

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("filer", "0011_auto_20190418_0137"),
        ("cms", "0022_auto_20180620_1551"),
        ("auth", "0009_alter_user_last_name_max_length"),
        ("courses", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="PageRole",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "role",
                    models.CharField(
                        choices=[("ADMIN", "Admin")],
                        help_text="A role describes all the permissions that should be granted to the user group.",
                        max_length=20,
                        verbose_name="role",
                    ),
                ),
                (
                    "folder",
                    models.OneToOneField(
                        blank=True,
                        editable=False,
                        help_text="Filer folder that this role controls.",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="role",
                        to="filer.Folder",
                        verbose_name="filer folder",
                    ),
                ),
                (
                    "group",
                    models.OneToOneField(
                        blank=True,
                        editable=False,
                        help_text="User group that this role controls.",
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="role",
                        to="auth.Group",
                        verbose_name="group",
                    ),
                ),
                (
                    "page",
                    models.ForeignKey(
                        help_text="Page to which this role grants permissions.",
                        limit_choices_to={"publisher_is_draft": True},
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="roles",
                        to="cms.Page",
                        verbose_name="page",
                    ),
                ),
            ],
            options={"verbose_name": "page role", "db_table": "richie_page_role"},
        ),
        migrations.AlterField(
            model_name="personpluginmodel",
            name="page",
            field=models.ForeignKey(
                limit_choices_to={"person__isnull": False, "publisher_is_draft": True},
                on_delete=django.db.models.deletion.CASCADE,
                related_name="person_plugins",
                to="cms.Page",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="pagerole", unique_together={("page", "role")}
        ),
    ]



"""Define a custom manager for page extensions"""
from django.apps import apps
from django.db import models
from django.utils import translation

from cms.extensions import PageExtension
from cms.models import Title
from cms.utils import get_current_site, i18n, page_permissions


def get_plugin_language_fallback_clause(language, is_draft):
    """
    Return a language fallback clause to apply in more complex queries where we
    select plugins and need to apply language fallbacks.
    """
    site = get_current_site()
    languages = [language] + i18n.get_fallback_languages(language, site_id=site.pk)
    languages = list(dict.fromkeys(languages))

    for item in range(len(languages)):
        for previous_item, previous_language in enumerate(languages[: item + 1]):

            qop_dict = {
                "extended_object__placeholders__cmsplugin__language": previous_language
            }
            if previous_item == item:
                if not is_draft:
                    qop_dict.update(
                        {
                            "extended_object__title_set__language": previous_language,
                            "extended_object__title_set__published": True,
                        }
                    )
                qop = models.Q(**qop_dict)
            else:
                qop = ~models.Q(**qop_dict)

            if previous_item == 0:
                subclause = qop
            else:
                subclause &= qop

        if item == 0:
            language_clause = subclause
        else:
            language_clause |= subclause

    return language_clause


def get_relevant_page_with_fallbacks(context, instance):
    """
    The plugin should show the published page whenever it exists or the draft page
    otherwise but only in edit mode.

    This is inspired by what DjangoCMS does in its main view:
    https://github.com/django-cms/django-cms/blob/3.8.0/cms/views.py#L37
    """
    request = context["request"]
    site = get_current_site()

    # Check permissions
    if not page_permissions.user_can_view_page(request.user, instance.page, site):
        return None

    relevant_page = instance.page.get_public_object()

    if not relevant_page:
        if context.get("current_page") and context["current_page"].publisher_is_draft:
            return instance.page
        return None

    if request.user.is_staff:
        user_languages = i18n.get_language_list(site_id=site.pk)
    else:
        user_languages = i18n.get_public_languages(site_id=site.pk)

    # These languages are then filtered out by the user allowed languages
    available_languages = [
        language
        for language in user_languages
        if language in list(relevant_page.get_published_languages())
    ]

    request_language = translation.get_language_from_request(request, check_path=True)
    if request_language not in user_languages:
        # Language is not allowed
        # Use the default site language
        default_language = i18n.get_default_language_for_site(site.pk)
        fallbacks = i18n.get_fallback_languages(default_language, site_id=site.pk)
        fallbacks = [default_language] + fallbacks
    else:
        fallbacks = i18n.get_fallback_languages(request_language, site_id=site.pk)

    # Only fallback to languages the user is allowed to see
    fallback_languages = [
        language
        for language in fallbacks
        if language != request_language and language in available_languages
    ]

    if request_language not in available_languages:
        if not fallback_languages:
            # There is no page with the requested language
            # and there's no configured fallbacks
            return None
        if request_language in relevant_page.get_languages():
            # The page was already published and later unpublished in the current
            # language. In this case we must not fallback to another language.
            return None

    return relevant_page


class PageExtensionQuerySet(models.QuerySet):
    """
    Add custom filters to the default Django queryset for page extensions.
    """

    def drafts(self):
        """
        Custom filter to get only the draft page extension instances based on the status of the
        linked DjangoCMS page.
        """
        return self.filter(extended_object__publisher_is_draft=True)

    def published(self):
        """
        Custom filter to get only the published page extension instances based on the status of the
        linked DjangoCMS page.
        """
        return self.filter(extended_object__publisher_is_draft=False)


class PageExtensionManager(models.Manager):
    """
    Add custom filters to the default Django manager for page extensions.
    """

    def get_queryset(self):
        """
        Use our custom queryset for page extensions.
        """
        return PageExtensionQuerySet(self.model, using=self._db)

    def drafts(self):
        """
        Make our custom filter "drafts" available on the page extension manager.
        """
        return self.get_queryset().drafts()

    def published(self):
        """
        Make our custom filter "published" available on the page extension manager.
        """
        return self.get_queryset().published()


class BasePageExtension(PageExtension):
    """
    The base page extension class is used as base for all our page extension models
    """

    objects = PageExtensionManager()

    class Meta:
        abstract = True

    def check_publication(self, language=None):
        """
        Allow checking if a page extension is published without passing any language (unlike the
        "is_published" method on the page object): if not explicitly passed as argument, the
        language is retrieved from the context.

        The actual check is subcontracted to the "is_published" method on the related Django CMS
        Page object.

        Note: We choose not to name our method "is_published" like Django CMS, because it is a
        bad practice. Indeed, someone may think it is a property and use it without invocating it
        and the returned value (a bound method) will always be truthy... This issue happened a lot
        with the "is_authenticated" method on Django's User model before they converted it to a
        property.
        """
        language = language or translation.get_language()
        return self.extended_object.is_published(language)

    def get_direct_related_page_extensions(
        self, extension_model, plugin_model, language=None
    ):
        """
        Return the page extensions linked to this page via a plugin in a placeholder, ranked by
        their `path` to respect the order in the page tree.
        """
        current_language = language or translation.get_language()
        site = get_current_site()

        languages = [current_language] + i18n.get_fallback_languages(
            current_language, site_id=site.pk
        )
        existing_languages = (
            plugin_model.objects.filter(
                cmsplugin_ptr__placeholder__page=self.extended_object
            )
            .values_list("cmsplugin_ptr__language", flat=True)
            .distinct()
        )

        relevant_language = next(
            filter(lambda l: l in existing_languages, languages), current_language
        )

        related_name = plugin_model.page.field.related_query_name()
        selector = f"extended_object__{related_name:s}__cmsplugin_ptr"
        # pylint: disable=no-member
        filter_dict = {
            f"{selector:s}__language": relevant_language,
            f"{selector:s}__placeholder__page": self.extended_object,
        }
        # For a public course, we must filter out page extensions that are not published
        # in any language
        if self.extended_object.publisher_is_draft is False:
            filter_dict["extended_object__title_set__published"] = True

        return (
            extension_model.objects.filter(**filter_dict)
            .select_related("extended_object")
            .order_by("extended_object__node__path")
            .distinct()
        )

    # pylint: disable=too-many-locals
    def get_reverse_related_page_extensions(
        self, model_name, language=None, include_descendants=False
    ):
        """
        Return a query to get the page extensions of a given model type related to the current
        page extension instance.

        For example: for an organization, it will return all courses that are pointing to this
        organization via an OrganizationPlugin in any placeholder of the course page.
        """
        is_draft = self.extended_object.publisher_is_draft
        # pylint: disable=no-member
        page_extension = self if is_draft else self.draft_extension
        page = page_extension.extended_object
        current_language = language or translation.get_language()
        language_clause = get_plugin_language_fallback_clause(
            current_language, is_draft
        )

        self_name = self._meta.model.__name__.lower()
        if include_descendants is True:
            bfs = (
                "extended_object__placeholders__cmsplugin__"
                f"courses_{self_name:s}pluginmodel__page__node"
            )
            selector = {
                f"{bfs:s}__path__startswith": page.node.path,
                f"{bfs:s}__depth__gte": page.node.depth,
            }
        else:
            bfs = (
                "extended_object__placeholders__cmsplugin__courses_"
                f"{self_name:s}pluginmodel__page"
            )
            selector = {bfs: page}

        # For a public page, we must filter out page extensions that are not published
        # in any language
        if is_draft is False:
            selector["extended_object__title_set__published"] = True

        page_extension_model = apps.get_model(
            app_label="courses", model_name=model_name
        )
        # pylint: disable=no-member
        return (
            page_extension_model.objects.filter(
                language_clause,
                extended_object__publisher_is_draft=is_draft,
                **selector,
            )
            .select_related("extended_object")
            .prefetch_related(
                models.Prefetch(
                    "extended_object__title_set",
                    to_attr="prefetched_titles",
                    queryset=Title.objects.filter(language=current_language),
                )
            )
            .distinct()
            .order_by("extended_object__node__path")
        )


class EsIdMixin:
    """
    Add a common method to get the ES ID for an indexed object to all the models
    that are indexed in ES.
    """

    def get_es_id(self):
        """
        Get the ElasticSearch ID for this object. It should be the same for both the published
        and draft versions of the object.
        """
        try:
            if self.public_extension:
                return str(self.public_extension.extended_object_id)
            if self.draft_extension:
                return str(self.extended_object_id)
        # pylint: disable=broad-except
        except AttributeError:
            pass
        return None



# Generated by Django 3.1.13 on 2021-08-11 10:34

from django.db import migrations
from django.utils.text import slugify


def normalize_course_code(apps, schema_editor):
    """
    Normalize existing course codes with the format expected as of today for all codes.
    """
    Course = apps.get_model("courses", "Course")

    for course in Course.objects.only("id", "code").iterator():
        Course.objects.filter(id=course.id).update(
            code=slugify(course.code, allow_unicode=True).upper()
            if course.code
            else None
        )


class Migration(migrations.Migration):

    dependencies = [("courses", "0030_auto_20210225_1435")]

    operations = [
        migrations.RunPython(normalize_course_code, migrations.RunPython.noop)
    ]



# Generated by Django 2.2.11 on 2020-03-13 14:14

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import cms.models.fields
import filer.fields.image


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
        migrations.swappable_dependency(settings.FILER_IMAGE_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Glimpse",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="glimpse_glimpse",
                        serialize=False,
                        to="cms.CMSPlugin",
                    ),
                ),
                ("title", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "variant",
                    models.CharField(
                        blank=True,
                        choices=[
                            (None, "Inherit"),
                            ("card_square", "Square card"),
                            ("row_half", "Half row"),
                            ("row_full", "Full row"),
                        ],
                        default=None,
                        help_text="Form factor variant",
                        max_length=50,
                        null=True,
                        verbose_name="Variant",
                    ),
                ),
                ("content", models.TextField(default="", verbose_name="Content")),
                (
                    "link_url",
                    models.URLField(
                        blank=True,
                        help_text="Make the glimpse as a link with an external URL.",
                        max_length=255,
                        null=True,
                        verbose_name="External URL",
                    ),
                ),
                (
                    "image",
                    filer.fields.image.FilerImageField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="image",
                        to=settings.FILER_IMAGE_MODEL,
                        verbose_name="image",
                    ),
                ),
                (
                    "link_page",
                    cms.models.fields.PageField(
                        blank=True,
                        help_text="Make the glimpse as a link with an internal (page) URL.",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="cms.Page",
                        verbose_name="Internal URL",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
    ]



"""
Customize text analysis & mapping to handle multilingual text and manage searches of word
fragments through n-grams.
"""

# Dynamic template to match translated strings with their language analyzer
# NB: expects objects to follow a conventional form, eg:
#   {
#       'title': {
#           'en': '<english title>',
#           'fr': '<french title>',
#       },
#   }
# Where all the language keys are optional and dynamically added to the mapping at indexing time
MULTILINGUAL_TEXT = [
    {
        "english_strings": {
            "match_mapping_type": "string",
            "path_match": "*.en",
            "mapping": {
                "type": "text",
                "fields": {
                    "language": {"type": "text", "analyzer": "english"},
                    "trigram": {
                        "type": "text",
                        "analyzer": "english_trigram",
                        # If we apply trigram on the text query, searching "artificial" will
                        # match "artificial" but also "art" with a lesser score. This is fine
                        # when results are sorted by score but in our case, sorting is complex
                        # and grouped in buckets (open, ongoing, coming soon, archived,...). For
                        # the moment, we consider that using a standard analyzer on the text
                        # query is good enough.
                        "search_analyzer": "english",
                    },
                },
            },
        }
    },
    {
        "french_strings": {
            "match_mapping_type": "string",
            "path_match": "*.fr",
            "mapping": {
                "type": "text",
                "fields": {
                    "language": {"type": "text", "analyzer": "french"},
                    "trigram": {
                        "type": "text",
                        "analyzer": "french_trigram",
                        # See above note explaining why we don't want to apply trigram on the
                        # text query.
                        "search_analyzer": "french",
                    },
                },
            },
        }
    },
]


# Settings inspired from
# https://www.elastic.co/guide/en/elasticsearch/reference/master/analysis-lang-analyzer.html
ANALYSIS_SETTINGS = {
    "analysis": {
        "filter": {
            "french_elision": {
                "type": "elision",
                "articles_case": True,
                "articles": [
                    "l",
                    "m",
                    "t",
                    "qu",
                    "n",
                    "s",
                    "j",
                    "d",
                    "c",
                    "jusqu",
                    "quoiqu",
                    "lorsqu",
                    "puisqu",
                ],
            },
            "french_stop": {"type": "stop", "stopwords": "_french_"},
            "french_stemmer": {"type": "stemmer", "language": "french"},
            "english_stop": {"type": "stop", "stopwords": "_english_"},
            "english_stemmer": {"type": "stemmer", "language": "english"},
            "english_possessive_stemmer": {
                "type": "stemmer",
                "language": "possessive_english",
            },
        },
        "analyzer": {
            "english": {
                "type": "custom",
                "tokenizer": "standard",
                "filter": [
                    "english_possessive_stemmer",
                    "asciifolding",
                    "lowercase",
                    "english_stop",
                    "english_stemmer",
                ],
            },
            "english_trigram": {
                "type": "custom",
                "tokenizer": "trigram",
                "filter": [
                    "english_possessive_stemmer",
                    "asciifolding",
                    "lowercase",
                    "english_stop",
                    "english_stemmer",
                ],
            },
            "french": {
                "type": "custom",
                "tokenizer": "standard",
                "filter": [
                    "french_elision",
                    "asciifolding",
                    "lowercase",
                    "french_stop",
                    "french_stemmer",
                ],
            },
            "french_trigram": {
                "type": "custom",
                "tokenizer": "trigram",
                "filter": [
                    "french_elision",
                    "asciifolding",
                    "lowercase",
                    "french_stop",
                    "french_stemmer",
                ],
            },
            "code": {
                "type": "custom",
                "tokenizer": "standard",
                "filter": [
                    "asciifolding",
                    "lowercase",
                ],
            },
            "code_trigram": {
                "type": "custom",
                "tokenizer": "trigram",
                "filter": [
                    "asciifolding",
                    "lowercase",
                ],
            },
            "simple_diacritics_insensitive": {
                "tokenizer": "lowercase",
                "filter": ["asciifolding"],
            },
        },
        "tokenizer": {
            "trigram": {
                "type": "ngram",
                "min_gram": 3,
                "max_gram": 20,
                "token_chars": ["letter", "digit"],
            }
        },
    },
    "max_ngram_diff": "20",
}



"""Richie template tags to manipulate categories."""
from django import template
from django.utils import translation

from classytags.arguments import Argument
from classytags.core import Options
from classytags.helpers import AsTag
from cms.models import Page

from ..models import Category

register = template.Library()


@register.tag("get_related_category_pages")
class GetRelatedCategories(AsTag):
    """
    A template tag to retrieve as a context variable, all the categories related via a
    CategoryPlugin to one or more of the pages targeted by a queryset or iterable.

    eg:
    {% get_related_category_pages pages as categories %}

    Keyword arguments:
        pages: a queryset of iterable targeting a group of cms pages.
    """

    options = Options(
        Argument("pages", required=True),
        "as",
        Argument("varname", required=True, resolve=False),
    )

    def get_value(self, context, **kwargs):
        """
        Get all categories related to either of the pages targeted by the queryset or
        iterator passed in argument.
        """
        current_page = context["current_page"]
        language = translation.get_language()

        selector = "category_plugins__cmsplugin_ptr__"
        filter_dict = {
            f"{selector:s}language": language,
            f"{selector:s}placeholder__page__publisher_is_draft": current_page.publisher_is_draft,
            f"{selector:s}placeholder__page__in": kwargs["pages"],
        }

        if context["current_page"].publisher_is_draft:
            query = Category.objects.filter(
                extended_object__publisher_is_draft=True,
                extended_object__in=Page.objects.filter(**filter_dict),
            )
        else:
            query = Category.objects.filter(
                extended_object__publisher_is_draft=False,
                extended_object__publisher_public__in=Page.objects.filter(
                    **filter_dict
                ),
            )

        return (
            query.select_related("extended_object")
            .order_by("extended_object__node__path")
            .distinct()
        )



# Generated by Django 2.2.16 on 2020-09-15 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("glimpse", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="glimpse",
            name="content",
            field=models.TextField(
                blank=True, default="", null=True, verbose_name="Content"
            ),
        ),
        migrations.AlterField(
            model_name="glimpse",
            name="variant",
            field=models.CharField(
                blank=True,
                choices=[
                    (None, "Inherit"),
                    ("card_square", "Square card"),
                    ("row_half", "Half row"),
                    ("row_full", "Full row"),
                    ("quote", "Quote"),
                ],
                default=None,
                help_text="Form factor variant",
                max_length=50,
                null=True,
                verbose_name="Variant",
            ),
        ),
    ]



"""Define SearchConfig as the default app configuration."""
from django.conf import settings

from .elasticsearch import (
    ElasticsearchClientCompat7to6,
    ElasticsearchIndicesClientCompat7to6,
)

# pylint: disable=invalid-name
default_app_config = "richie.apps.search.apps.SearchConfig"

ES_CLIENT = ElasticsearchClientCompat7to6(
    [getattr(settings, "RICHIE_ES_HOST", "elasticsearch")]
)

ES_INDICES_CLIENT = ElasticsearchIndicesClientCompat7to6(ES_CLIENT)



# Generated by Django 3.1.5 on 2021-01-19 13:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("courses", "0026_courserun_sync_mode")]

    operations = [
        migrations.AddField(
            model_name="personpluginmodel",
            name="bio",
            field=models.TextField(
                blank=True,
                default="",
                help_text=(
                    "Optional: provide a custom bio "
                    "(if you leave it empty, it will display the person's bio)."
                ),
                null=True,
                verbose_name="Custom bio",
            ),
        ),
        migrations.AlterField(
            model_name="course",
            name="code",
            field=models.CharField(
                blank=True,
                help_text="Unique reference for the course.",
                max_length=100,
                null=True,
                verbose_name="code",
            ),
        ),
    ]



"""
Section CMS plugin factories
"""
import factory

from .defaults import SECTION_TEMPLATES
from .models import Section


class SectionFactory(factory.django.DjangoModelFactory):
    """
    Factory to create random instances of Section for testing.
    """

    class Meta:
        model = Section

    title = factory.Faker("sentence", nb_words=8)
    template = SECTION_TEMPLATES[0][0]



"""Views for richie's search application."""
from django.contrib import messages
from django.core import management
from django.http import HttpResponse
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import cache_page

from rest_framework.decorators import api_view
from rest_framework.response import Response

from .filter_definitions import FILTERS


@api_view(["POST"])
# pylint: disable=unused-argument
def bootstrap_elasticsearch(request, version):
    """Regenerate the Elasticsearch index."""
    user = request.user
    if not (user.is_staff and request.user.has_perm("search.can_manage_elasticsearch")):
        return HttpResponse(
            force_str(_("You are not allowed to manage the search index.")),
            status=403 if request.user.is_authenticated else 401,
        )

    management.call_command("bootstrap_elasticsearch")

    messages.info(request, _("The search index was successfully bootstrapped"))
    return Response({})


@api_view(["GET"])
@cache_page(60 * 60 * 2)
# pylint: disable=unused-argument
def filter_definitions(request, version):
    """
    Make available on an API route the static parts of filter definitions.
    This is useful to some frontend components that need them to configure themselves.
    """
    return Response(
        {
            name: faceted_definition
            for filter in FILTERS.values()
            for name, faceted_definition in filter.get_static_definitions().items()
        }
    )



"""Serializers for Richie's courses app."""
from django.utils.functional import lazy

from rest_framework import serializers

from richie.apps.core.defaults import ALL_LANGUAGES

from .models import CourseRun


class ListMultipleChoiceField(serializers.MultipleChoiceField):
    """
    Override DRF's MultipleChoiceField to represent it as a list.
    We don't want choices to render as a set e.g. {"en", "fr"}
    """

    def to_representation(self, value):
        return sorted(list(super().to_representation(value)))


class CourseRunSerializer(serializers.ModelSerializer):
    """
    Course run serializer. Includes state but not any nested object.
    """

    languages = ListMultipleChoiceField(choices=lazy(lambda: ALL_LANGUAGES, tuple)())

    class Meta:
        model = CourseRun
        fields = [
            "id",
            "title",
            "resource_link",
            "start",
            "end",
            "enrollment_start",
            "enrollment_end",
            "languages",
            "state",
            "enrollment_count",
        ]


class SyncCourseRunSerializer(serializers.ModelSerializer):
    """
    Course run hook serializer.
    """

    languages = ListMultipleChoiceField(choices=lazy(lambda: ALL_LANGUAGES, tuple)())

    class Meta:
        model = CourseRun
        fields = [
            "resource_link",
            "start",
            "end",
            "enrollment_start",
            "enrollment_end",
            "languages",
            "enrollment_count",
        ]
        extra_kwargs = {"resource_link": {"required": True}}



"""Custom template tags for the search application of Richie."""
from django import template
from django.core.exceptions import ImproperlyConfigured

from ..defaults import RFC_5646_LOCALES

# pylint: disable=invalid-name
register = template.Library()


@register.filter
def rfc_5646_locale(value):
    """
    Converts a language (simple ISO639-1 or full language with regions) to a BCP47/RFC5646 locale.
    This will allow us to use this locale on the <html lang> attribute directly.
    """
    if "_" in value or "-" in value:
        value_locale = value[:2].lower() + "-" + value[3:].upper()  # noqa
    else:
        value_locale = value.lower()
    # pylint: disable=unsupported-membership-test
    if value_locale in RFC_5646_LOCALES:
        return value_locale
    # pylint: disable=not-an-iterable
    for locale in RFC_5646_LOCALES:
        if locale[:2] == value_locale:
            return locale
    raise ImproperlyConfigured(
        f"{value:s} does not correspond to any supported RFC 5646 locale."
    )



"""
Simple text CMS plugin
"""
from django.utils.translation import gettext_lazy as _

from cms.models import Placeholder
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from djangocms_text_ckeditor.widgets import TextEditorWidget

from richie.apps.core.defaults import PLUGINS_GROUP

from .defaults import SIMPLETEXT_CONFIGURATION
from .forms import CKEditorPluginForm
from .models import SimpleText
from .validators import HTMLMaxLengthValidator


@plugin_pool.register_plugin
class CKEditorPlugin(CMSPluginBase):
    """
    A plugin to add text with CKEditor widget.
    """

    allow_children = False
    cache = True
    disable_child_plugins = True
    form = CKEditorPluginForm
    model = SimpleText
    module = PLUGINS_GROUP
    name = _("Simple text")
    render_template = "richie/simple_text_ckeditor/simple_text.html"

    fieldsets = ((None, {"fields": ["body"]}),)

    def get_form(self, request, obj=None, change=False, **kwargs):
        """
        Add an HTML max length validator and/or a CKEditor configuration based on what is
        configured in settings for the current placeholder.
        """
        form = super().get_form(request, obj=obj, change=change, **kwargs)

        placeholder_id = request.GET.get("placeholder_id")
        if not placeholder_id and not obj:
            return form

        if placeholder_id:
            placeholder = Placeholder.objects.only("slot").get(id=placeholder_id)
        else:
            placeholder = obj.placeholder

        for configuration in SIMPLETEXT_CONFIGURATION:
            if (
                "placeholders" not in configuration
                or placeholder.slot in configuration["placeholders"]
            ):
                break
        else:
            configuration = {}

        body_field = form.base_fields["body"]
        if configuration.get("max_length"):
            body_field.validators.append(
                HTMLMaxLengthValidator(configuration["max_length"])
            )

        if configuration.get("ckeditor"):
            body_field.widget = TextEditorWidget(
                configuration=configuration["ckeditor"]
            )

        return form

    def render(self, context, instance, placeholder):
        """
        Build plugin context passed to its template to perform rendering
        """
        context = super().render(context, instance, placeholder)
        context.update({"instance": instance})
        return context



"""
Glimpse plugin default settings
"""
from django.conf import settings
from django.utils.translation import gettext_lazy as _

BADGE, CARD_SQUARE, PERSON, QUOTE, ROW_HALF, ROW_FULL = (
    "badge",
    "card_square",
    "person",
    "quote",
    "row_half",
    "row_full",
)

GLIMPSE_VARIANTS = getattr(
    settings,
    "RICHIE_GLIMPSE_VARIANTS",
    [
        (None, _("Inherit")),
        (BADGE, _("Badge")),
        (CARD_SQUARE, _("Square card")),
        (PERSON, _("Person")),
        (QUOTE, _("Quote")),
        (ROW_HALF, _("Half row")),
        (ROW_FULL, _("Full row")),
    ],
)



"""
LTI consumer CMS plugin
"""
from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from richie.apps.core.defaults import PLUGINS_GROUP

from .forms import LTIConsumerForm
from .models import LTIConsumer


@plugin_pool.register_plugin
class LTIConsumerPlugin(CMSPluginBase):
    """
    A plugin to consume LTI content.
    """

    allow_children = False
    cache = True
    disable_child_plugins = True
    form = LTIConsumerForm
    model = LTIConsumer
    module = PLUGINS_GROUP
    name = _("LTI consumer")
    render_template = "richie/lti_consumer/lti_consumer.html"

    class Media:
        """
        Simple UX improvement that hides useless fields if a predefined LTI provider is used
        """

        js = ("lti_consumer/js/change_form.js",)



"""
Helpers to enable reuse for needs that are shared between viewsets.
"""
from django.utils.translation import get_language_from_request

from rest_framework.decorators import action
from rest_framework.response import Response

from .. import ES_CLIENT


class ViewSetMetadata:
    """
    "Meta" class intended to be used as an attribute on ViewSets to provide a common set of
    meta attributes, like the Django `class Meta` on eg. models.
    """

    def __init__(self, indexer):
        """
        Set the required attributes passed from the call site. Currently this only includes
        the ViewSet's corresponding indexer.
        """
        self.indexer = indexer


class AutocompleteMixin:
    """
    Add a `/{resource}/autocomplete` route on a ViewSet enabling clients to make autocompletion
    requests using the specific fields & queries in ElasticSearch, as defined in the relevant
    resource's indexer.
    """

    # pylint: disable=unused-argument
    @action(detail=False)
    def autocomplete(self, request, version):
        """
        Use the "completion" field on the organization mapping & objects to provide autocomplete
        functionality through an API endpoint.
        """
        # This mixin is intended to be used on ViewSets. It requires a _meta attribute holding
        # the relevant indexer
        indexer = self._meta.indexer

        # Some malformed requests have been generating 500 errors in autocomplete calls. Make sure
        # we return the proper error code instead so the frontend can be debugged as well if
        # necessary.
        try:
            query = request.query_params["query"]
        except KeyError:
            return Response(
                status=400,
                data={
                    "errors": [
                        (
                            'Missing autocomplete "query" for request '
                            f"to {self._meta.indexer.index_name}."
                        )
                    ]
                },
            )

        # Query our specific ES completion field
        language = get_language_from_request(request)
        autocomplete_query_response = ES_CLIENT.search(
            index=indexer.index_name,
            body={
                "suggest": {
                    "objects": {
                        "prefix": query,
                        "completion": {"field": f"complete.{language:s}"},
                    }
                }
            },
        )

        # Build a response array from the list of completion options
        return Response(
            [
                indexer.format_es_document_for_autocomplete(
                    option, get_language_from_request(request)
                )
                for option in autocomplete_query_response["suggest"]["objects"][0][
                    "options"
                ]
            ]
        )



"""
Custom fields for the course application
"""
from datetime import datetime, time

from django.forms import SplitDateTimeField
from django.forms.utils import from_current_timezone


class CourseRunSplitDateTimeField(SplitDateTimeField):
    """
    A SplitDateTime field that ignores values when only the time is set and not the date. If only
    the date is set, we use a default time value.
    """

    def has_changed(self, initial, data):
        """
        We must consider that the field has not changed, and therefore should be ignored if the
        time is our default and the date has not been set.
        """
        result = super().has_changed(initial, data)
        if initial is None and not data[0] and data[1]:
            return False
        return result

    def compress(self, data_list):
        """
        On the DateTime widget, we don't want the time to be required, only the date:
        - if the time is not set on a form, we set it to 0:00:00,
        - if only the time is set, we ignore the value.
        """
        if data_list:
            if data_list[0] in self.empty_values:
                # If there is no date, we ignore the time and act as if nothing was submitted
                return None
            if data_list[1] in self.empty_values:
                # If there is a date but no time, we take midnight as default time
                data_list[1] = time(0)
            result = datetime.combine(*data_list)
            return from_current_timezone(result)
        return None



# Generated by Django 3.2.7 on 2021-09-30 11:10

from django.db import migrations

import djangocms_attributes_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ("section", "0005_migrate_sectionlist_to_nesteditem"),
    ]

    operations = [
        migrations.AddField(
            model_name="section",
            name="attributes",
            field=djangocms_attributes_field.fields.AttributesField(
                blank=True, default=dict, verbose_name="Attributes"
            ),
        ),
    ]



"""
NestedItem plugin models
"""
from django.db import models
from django.utils.text import Truncator
from django.utils.translation import gettext_lazy as _

from cms.models.pluginmodel import CMSPlugin

from .defaults import NESTEDITEM_VARIANTS


class NestedItem(CMSPlugin):
    """
    Nested item
    """

    content = models.TextField(_("Content"), blank=True, default="")
    variant = models.CharField(
        _("Variant"),
        max_length=50,
        choices=NESTEDITEM_VARIANTS,
        default=NESTEDITEM_VARIANTS[0][0],
        help_text=_("Form factor variant"),
    )

    def __str__(self):
        return Truncator(self.content).words(6, truncate="...")



"""
Section plugin models
"""
from django.db import models
from django.utils.text import Truncator
from django.utils.translation import gettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from djangocms_attributes_field.fields import AttributesField

from .defaults import SECTION_TEMPLATES


class Section(CMSPlugin):
    """
    Section plugin model

    A Section aims to enforce a distinct title from its content

    It should be used as filling a title then adding sub content plugins.
    """

    title = models.CharField(max_length=255, blank=True, null=True)
    template = models.CharField(
        _("Template"),
        max_length=150,
        choices=SECTION_TEMPLATES,
        default=SECTION_TEMPLATES[0][0],
        help_text=_("Optional template for custom look."),
    )
    attributes = AttributesField(
        verbose_name=_("Attributes"),
        blank=True,
        excluded_keys=["href", "target"],
    )

    def __str__(self):
        return Truncator(self.title).words(6, truncate="...")



"""
Import custom settings and set up defaults for values the Search app needs
"""
from django.conf import settings
from django.utils.functional import lazy
from django.utils.translation import gettext_lazy as _

# Elasticsearch
ES_CHUNK_SIZE = 500
ES_PAGE_SIZE = 10

# Use a lazy to enable easier testing by not defining the value at bootstrap time
ES_INDICES_PREFIX = lazy(lambda: settings.RICHIE_ES_INDICES_PREFIX)()

# Define which analyzer should be used for each language
QUERY_ANALYZERS = getattr(
    settings, "RICHIE_QUERY_ANALYZERS", {"en": "english", "fr": "french"}
)

# Define the scoring boost (in ElasticSearch) related value names receive when using
# full-text search.
# For example, when a user searches for "Science" in full-text, it should match any
# course whose category contains "Science" or a related word, albeit with a lower
# score than courses that include it in their title or description.
# This lower score factor is the boost value we get or set here.
RELATED_CONTENT_BOOST = 0.05

FACET_SORTING_DEFAULT = "count"

FACET_COUNTS_DEFAULT_LIMIT = getattr(settings, "RICHIE_FACET_COUNTS_DEFAULT_LIMIT", 10)
FACET_COUNTS_MAX_LIMIT = getattr(settings, "RICHIE_FACET_COUNTS_MAX_LIMIT", 50)

ES_STATE_WEIGHTS = getattr(settings, "RICHIE_ES_STATE_WEIGHTS", None) or [
    80,  # ONGOING_OPEN
    70,  # FUTURE_OPEN
    60,  # ARCHIVED_OPEN
    30,  # FUTURE_NOT_YET_OPEN
    6,  # FUTURE_CLOSED
    5,  # ONGOING_CLOSED
    1,  # ARCHIVED_CLOSED
]

FILTERS_CONFIGURATION = [
    (
        "richie.apps.search.filter_definitions.StaticChoicesFilterDefinition",
        {
            "fragment_map": {"new": [{"term": {"is_new": True}}]},
            "human_name": _("New courses"),
            "min_doc_count": 0,
            "name": "new",
            "position": 0,
            "sorting": "conf",
            "values": {"new": _("First session")},
        },
    ),
    (
        "richie.apps.search.filter_definitions.NestingWrapper",
        {
            "name": "course_runs",
            "filters": [
                (
                    "richie.apps.search.filter_definitions.AvailabilityFilterDefinition",
                    {
                        "human_name": _("Availability"),
                        "is_drilldown": True,
                        "min_doc_count": 0,
                        "name": "availability",
                        "position": 1,
                        "sorting": "conf",
                    },
                ),
                (
                    "richie.apps.search.filter_definitions.LanguagesFilterDefinition",
                    {
                        "human_name": _("Languages"),
                        # There are too many available languages to show them all, all the time.
                        # Eg. 200 languages, 190+ of which will have 0 matching courses.
                        "min_doc_count": 1,
                        "name": "languages",
                        "position": 5,
                    },
                ),
            ],
        },
    ),
    (
        "richie.apps.search.filter_definitions.IndexableHierarchicalFilterDefinition",
        {
            "human_name": _("Subjects"),
            "is_autocompletable": True,
            "is_searchable": True,
            "min_doc_count": 0,
            "name": "subjects",
            "position": 2,
            "reverse_id": "subjects",
            "term": "categories",
        },
    ),
    (
        "richie.apps.search.filter_definitions.IndexableHierarchicalFilterDefinition",
        {
            "human_name": _("Levels"),
            "is_autocompletable": True,
            "is_searchable": True,
            "min_doc_count": 0,
            "name": "levels",
            "position": 3,
            "reverse_id": "levels",
            "term": "categories",
        },
    ),
    (
        "richie.apps.search.filter_definitions.IndexableHierarchicalFilterDefinition",
        {
            "human_name": _("Organizations"),
            "is_autocompletable": True,
            "is_searchable": True,
            "min_doc_count": 0,
            "name": "organizations",
            "position": 4,
            "reverse_id": "organizations",
        },
    ),
    (
        "richie.apps.search.filter_definitions.IndexableFilterDefinition",
        {
            "human_name": _("Persons"),
            "is_autocompletable": True,
            "is_searchable": True,
            "min_doc_count": 0,
            "name": "persons",
            "position": 5,
            "reverse_id": "persons",
        },
    ),
]



"""
Template context processors
"""
import json
from collections import OrderedDict

from django.conf import settings
from django.contrib.sites.models import Site
from django.http.request import HttpRequest
from django.middleware.csrf import get_token
from django.utils.translation import get_language_from_request

from richie.apps.courses.models import Organization

from . import defaults


def site_metas(request: HttpRequest):
    """
    Context processor to add all information required by Richie CMS templates and frontend.

    If `CDN_DOMAIN` settings is defined we add it in the context. It allows
    to load statics js on a CDN like cloudfront.
    """
    site_current = Site.objects.get_current()
    protocol = "https" if request.is_secure() else "http"

    context = {
        **{
            f"GLIMPSE_PAGINATION_{k.upper()}": v
            for k, v in {
                **defaults.GLIMPSE_PAGINATION,
                **getattr(settings, "RICHIE_GLIMPSE_PAGINATION", {}),
            }.items()
        },
        "SITE": {
            "name": site_current.name,
            "domain": site_current.domain,
            "web_url": f"{protocol:s}://{site_current.domain:s}",
        },
        "FRONTEND_CONTEXT": {
            "context": {
                "csrftoken": get_token(request),
                "environment": getattr(settings, "ENVIRONMENT", ""),
                "release": getattr(settings, "RELEASE", ""),
                "sentry_dsn": getattr(settings, "SENTRY_DSN", ""),
            }
        },
        **WebAnalyticsContextProcessor().context_processor(request),
    }

    if getattr(settings, "CDN_DOMAIN", None):
        context["CDN_DOMAIN"] = settings.CDN_DOMAIN

    # Add a MEDIA_URL_PREFIX to context to prefix the media url files to have an absolute URL
    if settings.MEDIA_URL.startswith("//"):
        # Eg. //my-cdn-user.cdn-provider.com/media/
        context["MEDIA_URL_PREFIX"] = f"{request.scheme:s}:"
    elif settings.MEDIA_URL.startswith("/"):
        # Eg. /media/
        context["MEDIA_URL_PREFIX"] = f"{protocol:s}://{site_current.domain:s}"
    else:
        # Eg. https://my-cdn-user.cdn-provider.com/media/
        context["MEDIA_URL_PREFIX"] = ""

    authentication_delegation = getattr(
        settings, "RICHIE_AUTHENTICATION_DELEGATION", None
    )
    if authentication_delegation:

        context["AUTHENTICATION"] = {
            "profile_urls": json.dumps(
                {
                    key: {
                        "label": str(url["label"]),
                        "action": str(
                            url["href"].format(
                                base_url=authentication_delegation["BASE_URL"]
                            )
                        ),
                    }
                    for key, url in authentication_delegation.get(
                        "PROFILE_URLS", {}
                    ).items()
                }
            ),
        }

        context["FRONTEND_CONTEXT"]["context"]["authentication"] = {
            "endpoint": authentication_delegation["BASE_URL"],
            "backend": authentication_delegation["BACKEND"],
        }

    if getattr(settings, "RICHIE_LMS_BACKENDS", None):
        context["FRONTEND_CONTEXT"]["context"]["lms_backends"] = [
            {
                "endpoint": lms["BASE_URL"],
                "backend": lms["JS_BACKEND"],
                "course_regexp": lms["JS_COURSE_REGEX"],
            }
            for lms in getattr(settings, "RICHIE_LMS_BACKENDS", [])
        ]

    context["FRONTEND_CONTEXT"] = json.dumps(context["FRONTEND_CONTEXT"])

    if getattr(settings, "RICHIE_MINIMUM_COURSE_RUNS_ENROLLMENT_COUNT", None):
        context[
            "RICHIE_MINIMUM_COURSE_RUNS_ENROLLMENT_COUNT"
        ] = settings.RICHIE_MINIMUM_COURSE_RUNS_ENROLLMENT_COUNT
    return context


class WebAnalyticsContextProcessor:
    """
    Context processor to add Web Analytics tracking information to Richie CMS templates and
    frontend.
    """

    def context_processor(self, request: HttpRequest) -> dict:
        """
        Real implementation of the context processor for the Web Analytics core app sub-module
        """
        context = {}
        if hasattr(request, "current_page"):
            # load web analytics settings to the context
            if getattr(settings, "WEB_ANALYTICS_ID", None):
                context["WEB_ANALYTICS_ID"] = settings.WEB_ANALYTICS_ID
                context["WEB_ANALYTICS_DIMENSIONS"] = self.get_dimensions(request)

            context["WEB_ANALYTICS_LOCATION"] = getattr(
                settings, "WEB_ANALYTICS_LOCATION", "head"
            )

            context["WEB_ANALYTICS_PROVIDER"] = getattr(
                settings, "WEB_ANALYTICS_PROVIDER", "google_analytics"
            )
        return context

    # pylint: disable=no-self-use
    def get_dimensions(self, request: HttpRequest) -> dict:
        """
        Compute the web analytics dimensions (dict) that would be added to the Django context
        They are a dictionary like:
        ```
        {
            "organizations_codes": ["UNIV_LISBON", "UNIV_PORTO"],
            "course_code": ["COURSE_XPTO"],
            "course_runs_titles": [
                "Summer edition",
                "Winter edition"
            ],
            "course_runs_resource_links": [
                "http://example.edx:8073/courses/course-v1:edX+DemoX+Demo_Course/info",
                "http://example.edx:8073/courses/course-v1:edX+DemoX+Demo_Course_2/info"
            ],
            "page_title": ["Introduction to Programming"],
        }
        ```

        Args:
            request (HttpRequest): The Http request
            web_analytics_context (dict): the context relevant for the web analytics sub module

        Returns:
            dict: a dict with the dimensions, where each value is a list
        """
        # Warn do not change the order of evaluation of this methods, because on Google Analytics
        # the key is a 'dimension1' instead of a more generic key like 'organizations'.
        # That's why we are using an OrderedDict instead of a normal Python dict (don't support
        # ordering)
        dimensions = OrderedDict()

        page = request.current_page or None
        language = get_language_from_request(request, check_path=True)

        organizations_codes = []
        if page and not page.is_home:
            organizations_codes = Organization.get_organizations_codes(page, language)
        dimensions["organizations_codes"] = organizations_codes

        course = getattr(page, "course", None)
        dimensions["course_code"] = [getattr(course, "code", "")]

        course_runs = course.course_runs if course else []
        dimensions["course_runs_titles"] = [
            course_run.title
            for course_run in course_runs
            if course_run is not None and course_run.safe_title is not None
        ]

        dimensions["course_runs_resource_links"] = map(
            lambda course_run: course_run.resource_link, course_runs
        )

        dimensions["page_title"] = [page.get_title() if page else ""]
        return dimensions



"""
Toolbar extension for the search application
"""
from django.utils.translation import gettext_lazy as _

from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from rest_framework.reverse import reverse


@toolbar_pool.register
class SearchToolbar(CMSToolbar):
    """Customize the toolbar to add a button that regenerates the search index."""

    def populate(self):
        """
        Show an item in the toolbar only to users who have the permission to regenerate the
        search index.
        """

        if self.request.user.has_perm("search.can_manage_elasticsearch"):
            # Get the action endpoint for API version 1. We hardcode it as only this
            # piece of code will determine when to change the endpoint version.
            url = reverse("bootstrap_elasticsearch", kwargs={"version": "1.0"})
            admin_menu = self.toolbar.get_menu(ADMIN_MENU_IDENTIFIER)

            # Create the new menu item as an Ajax call
            admin_menu.add_ajax_item(
                _("Regenerate search index..."),
                action=url,
                data={},
                on_success=self.toolbar.REFRESH_PAGE,
                position=3,
            )



# Generated by Django 3.2.7 on 2021-10-04 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("courses", "0031_auto_20210811_1234"),
    ]

    operations = [
        migrations.AddField(
            model_name="courserun",
            name="enrollment_count",
            field=models.PositiveIntegerField(
                default=0,
                help_text="The number of enrolled students",
                verbose_name="enrollment count",
            ),
        ),
    ]



"""
Large banner CMS plugin factories
"""
import factory

from .defaults import LARGEBANNER_TEMPLATES
from .models import LargeBanner


class LargeBannerFactory(factory.django.DjangoModelFactory):
    """
    Factory to create random instances of LargeBanner for testing.
    """

    class Meta:
        model = LargeBanner

    title = factory.Faker("sentence", nb_words=8)
    background_image = factory.SubFactory(
        "richie.apps.core.factories.FilerImageFactory"
    )
    logo = factory.SubFactory("richie.apps.core.factories.FilerImageFactory")
    logo_alt_text = factory.Faker("sentence", nb_words=5)
    content = factory.Faker("text", max_nb_chars=42)
    template = LARGEBANNER_TEMPLATES[0][0]



"""
Plugin for django-redis that supports redis sentinel.
Credits :
 - https://github.com/lamoda/django-sentinel
 - https://github.com/KabbageInc/django-redis-sentinel
"""

import logging
import random

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.cache import get_max_age, patch_response_headers
from django.utils.deprecation import MiddlewareMixin

from django_redis.client import DefaultClient
from redis.sentinel import Sentinel

DJANGO_REDIS_LOGGER = getattr(settings, "DJANGO_REDIS_LOGGER", __name__)

logger = logging.getLogger(__name__)


class SentinelClient(DefaultClient):
    """
    Sentinel client object extending django-redis DefaultClient
    """

    def __init__(self, server, params, backend):
        """
        Slightly different logic than connection to multiple Redis servers.
        Reserve only one write and one read descriptor, as they will be closed on exit anyway.
        """
        super().__init__(server, params, backend)
        self._client_write = None
        self._client_read = None
        (
            self._master_name,
            self._sentinel_hosts,
            self._database_name,
        ) = self.parse_connection_string(server)
        self.log = logging.getLogger(DJANGO_REDIS_LOGGER)

    @staticmethod
    def parse_connection_string(connection_string):
        """
        Parse connection string in format:
            master_name/sentinel_server:port,sentinel_server:port/db_id
        Returns master name, list of tuples with pair (host, port) and db_id
        """
        try:
            master_name, servers_string, database_name = connection_string.split("/")
            servers = [host_port.split(":") for host_port in servers_string.split(",")]
            sentinel_hosts = [(host, int(port)) for host, port in servers]
        except (ValueError, TypeError, IndexError) as error:
            raise ImproperlyConfigured(
                f"Incorrect format '{connection_string:s}'"
            ) from error

        return master_name, sentinel_hosts, database_name

    def get_client(self, write=True, tried=(), show_index=False):
        """
        Method used to obtain a raw redis client.

        This function is used by almost all cache backend
        operations to obtain a native redis client/connection
        instance.
        """
        self.log.debug("get_client called: write=%s", write)
        if write:
            if self._client_write is None:
                self._client_write = self.connect(write)

            if show_index:
                return self._client_write, 0
            return self._client_write

        if self._client_read is None:
            self._client_read = self.connect(write)

        if show_index:
            return self._client_read, 0
        return self._client_read

    # pylint: disable=arguments-renamed
    def connect(self, write=True):
        """
        Create a redis connection with connection pool.
        """
        self.log.debug("connect called: write=%s", write)

        sentinel_timeout = self._options.get("SENTINEL_TIMEOUT", 1)
        password = self._options.get("PASSWORD", None)
        sentinel = Sentinel(
            self._sentinel_hosts, socket_timeout=sentinel_timeout, password=password
        )

        if write:
            host, port = sentinel.discover_master(self._master_name)
        else:
            try:
                host, port = random.choice(  # nosec
                    sentinel.discover_slaves(self._master_name)
                )
            except IndexError:
                self.log.debug("no slaves are available. using master for read.")
                host, port = sentinel.discover_master(self._master_name)

        if password:
            connection_url = f"redis://:{password}@{host}:{port}/{self._database_name}"
        else:
            connection_url = f"redis://{host}:{port}/{self._database_name}"
        self.log.debug("Connecting to: %s", connection_url)
        return self.connection_factory.connect(connection_url)

    def close(self, **kwargs):
        """
        Closing old connections, as master may change in time of inactivity.
        """
        self.log.debug("close called")
        if self._client_read:
            # pylint: disable=protected-access
            for connection in self._client_read.connection_pool._available_connections:
                connection.disconnect()
            self.log.debug("client_read closed")

        if self._client_write:
            # pylint: disable=protected-access
            for connection in self._client_write.connection_pool._available_connections:
                connection.disconnect()
            self.log.debug("client_write closed")

        del self._client_write
        del self._client_read
        self._client_write = None
        self._client_read = None


class LimitBrowserCacheTTLHeaders(MiddlewareMixin):
    """
    This middleware allows to define a maximum cache timeout for the cache
    response headers (Cache-control: max-age and Expires).

    It can be configured with the MAX_BROWSER_CACHE_TTL setting.
    This value is expressed in seconds.

    LimitBrowserCacheTTLHeaders should be placed at the beginning of the
    MIDDLEWARE list, so that it'll get called last during the response phase.
    """

    # pylint: disable=no-self-use
    def process_response(self, request, response):
        """
        Rewrite the "Cache-control" and "Expires headers" in the response
        if needed.
        """
        try:
            max_ttl = int(getattr(settings, "MAX_BROWSER_CACHE_TTL", None))
            if max_ttl < 0:
                raise ValueError("MAX_BROWSER_CACHE_TTL must be a positive integer")
        except (ValueError, TypeError) as err:
            logger.error(err)
            return response

        max_age = get_max_age(response)
        if max_age is not None and max_age > max_ttl:
            if response.has_header("Expires"):
                # Remove the Expires response Header because patch_response_headers()
                # adds it only if it isn't already set
                del response["Expires"]
            patch_response_headers(response, cache_timeout=max_ttl)

        return response



# Generated by Django 2.2.9 on 2020-01-02 15:07

import django.db.models.deletion
from django.db import migrations, models

import parler.models

import richie.apps.core.fields.multiselect


class Migration(migrations.Migration):

    dependencies = [("courses", "0011_deprecate_untranslated_licence_fields")]

    operations = [
        migrations.CreateModel(
            name="LicenceTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                ("name", models.CharField(max_length=200, verbose_name="name")),
                ("content", models.TextField(default="", verbose_name="content")),
                (
                    "master",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="courses.Licence",
                    ),
                ),
            ],
            options={
                "verbose_name": "Licence translation",
                "db_table": "richie_licence_translation",
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        )
    ]



"""
API Routes exposed by our Search app
"""
from django.urls import path

from rest_framework import routers

from .views import bootstrap_elasticsearch, filter_definitions
from .viewsets.categories import CategoriesViewSet
from .viewsets.courses import CoursesViewSet
from .viewsets.organizations import OrganizationsViewSet
from .viewsets.persons import PersonsViewSet

# Instantiate a router
ROUTER = routers.SimpleRouter()

# Define our app's routes with the router
ROUTER.register(r"courses", CoursesViewSet, "courses")
ROUTER.register(r"organizations", OrganizationsViewSet, "organizations")
ROUTER.register(r"persons", PersonsViewSet, "persons")
ROUTER.register(r"(?P<kind>\w+)", CategoriesViewSet, "categories")

# Use the standard name for our urlpatterns so urls.py can import it effortlessly
urlpatterns = [
    path(
        r"bootstrap-elasticsearch/",
        bootstrap_elasticsearch,
        name="bootstrap_elasticsearch",
    ),
    path(r"filter-definitions/", filter_definitions, name="filter_definitions"),
]

urlpatterns += ROUTER.urls



"""
Build an interoperability layer that allows Richie to work with ElasticSearch 6
just like it does with ElasticSearch 7.
"""
# pragma pylint: disable=W0221
from django.utils.functional import cached_property

from elasticsearch import Elasticsearch, Transport
from elasticsearch.client import IndicesClient
from elasticsearch.helpers import bulk

# Dummy type used to satisfy the ES6 requirement to have type. "_doc" is conventional,
# and the actual value of the string does not change anything functionally.
DOC_TYPE = "_doc"


class ElasticsearchClientCompat7to6(Elasticsearch):
    """
    Compatibility wrapper around the Elasticsearch client from elasticsearch-py that
    handles incompatibilities to let Richie run ES6 and ES7.
    """

    def __init__(self, hosts=None, transport_class=Transport, **kwargs):
        """
        Instantiate the actual Elasticsearch client, then use it to detect the version
        of Elasticsearch we're working with.
        """
        super().__init__(hosts=hosts, transport_class=transport_class, **kwargs)

    @cached_property
    def __es_version__(self):
        """
        First retrieve version from elasticsearch server then returns the cached result
        """
        return self.info()["version"]["number"][:1]

    # pylint: disable=W0622
    def get(self, index, id):
        """
        Patch the dummy doc type onto document retrieval requests so ES6 accepts the requests.
        """

        return super().get(index=index, id=id, doc_type=DOC_TYPE)

    def search(self, body=None, index=None, params=None, **kwargs):
        """
        Patch the "value" & "relation" dict in place of the int returned by ES6 for
        search query hits total count.
        """

        search_response = super().search(
            body=body, index=index, params=params or {}, **kwargs
        )

        if self.__es_version__ == "6":
            search_response["hits"]["total"] = {
                "value": search_response["hits"]["total"],
                "relation": "eq",
            }

        return search_response


class ElasticsearchIndicesClientCompat7to6(IndicesClient):
    """
    Compatibility wrapper around the ES IndicesClient from elasticsearch-py that
    handles incompatibilities to let Richie run ES6 and ES7.
    """

    def get_mapping(self, index=None, params=None):
        """
        Pluck from the dummy type in the mapping if using ES6, which nests the actual
        mapping info under the document type.
        """

        mappings = super().get_mapping(index=index, params=params or {})

        if self.client.__es_version__ == "6":
            for index_name in mappings.keys():
                mappings[index_name]["mappings"] = mappings[index_name]["mappings"][
                    DOC_TYPE
                ]

        return mappings

    def put_mapping(self, body, index=None, params=None):
        """
        Add our dummy type in kwargs of the put_mapping call to satisfy the requirement in ES.
        """

        if self.client.__es_version__ == "6":
            return super().put_mapping(
                body, doc_type=DOC_TYPE, index=index, params=params or {}
            )

        return super().put_mapping(body, index=index, params=params or {})


def bulk_compat_7_to_6(client, actions, *args, stats_only=False, **kwargs):
    """
    Patch a dummy type on all actions to satisfy the requirement for ES6. As types are not
    actually used for anything, we can use the same value everywhere.
    """
    if client.__es_version__ == "6":
        # Use a generator expression instead of a for loop to keep actions lazy
        actions = ({**action, "_type": DOC_TYPE} for action in actions)

    bulk(client, actions, stats_only=stats_only, *args, **kwargs)


bulk_compat = bulk_compat_7_to_6



# Generated by Django 3.1.7 on 2021-02-25 13:35

from math import ceil

from django.db import migrations

from ..defaults import DAY, HOUR, MINUTE, MONTH, TIME_UNITS, WEEK


def populate_effort(apps, schema_editor):
    """
    Populate the "effort" field by crossing information that
    we have from "duration" and "effort_deprecated" fields
    """
    Course = apps.get_model("courses", "Course")

    TIME_UNITS_IN_SECONDS = {
        MINUTE: 60,
        HOUR: 60 * 60,
        DAY: 24 * 60 * 60,
        WEEK: 7 * 24 * 60 * 60,
        MONTH: 30 * 24 * 60 * 60,
    }

    for course in Course.objects.iterator():

        if course.duration is not None and course.effort_deprecated is not None:
            (duration, duration_unit) = course.duration
            (
                effort_deprecated,
                effort_deprecated_unit,
                effort_deprecated_reference_unit,
            ) = course.effort_deprecated
            duration_in_seconds = duration * TIME_UNITS_IN_SECONDS[duration_unit]
            effort_in_seconds = int(
                (
                    effort_deprecated
                    / TIME_UNITS_IN_SECONDS[effort_deprecated_reference_unit]
                )
                * duration_in_seconds
                * TIME_UNITS_IN_SECONDS[effort_deprecated_unit]
            )

            if effort_in_seconds > TIME_UNITS_IN_SECONDS[HOUR]:
                new_effort = (
                    ceil(effort_in_seconds / TIME_UNITS_IN_SECONDS[HOUR]),
                    HOUR,
                )
            else:
                new_effort = (
                    ceil(effort_in_seconds / TIME_UNITS_IN_SECONDS[MINUTE]),
                    MINUTE,
                )

            course.effort = new_effort
            course.save()


def reverse_populate_effort(apps, schema_editor):
    """
    Rollback to old effort field values

    As new pace course is the old effort field meaning,
    we try to compute course pace and set to effort_deprecated field.
    """
    Course = apps.get_model("courses", "Course")

    for course in Course.objects.iterator():
        if course.effort is not None and course.duration is not None:
            (effort, effort_unit) = course.effort
            (duration, duration_unit) = course.duration

            # Ensure that units are compatible with EffortField requirement
            time_units_keys = list(dict(TIME_UNITS))
            if time_units_keys.index(effort_unit) < time_units_keys.index(
                duration_unit
            ):
                pace = int(effort / duration)
                course.effort_deprecated = (pace, effort_unit, duration_unit)
                course.save()


class Migration(migrations.Migration):

    dependencies = [
        ("courses", "0029_auto_20210225_1429"),
    ]

    operations = [
        migrations.RunPython(populate_effort, reverse_populate_effort),
        migrations.RemoveField(model_name="course", name="effort_deprecated"),
    ]



"""
Plain text CMS plugin
"""
from django.conf import settings
from django.core.validators import MaxLengthValidator
from django.utils.translation import gettext_lazy as _

from cms.models import Placeholder
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from richie.apps.core.defaults import PLUGINS_GROUP

from .models import PlainText


@plugin_pool.register_plugin
class PlainTextPlugin(CMSPluginBase):
    """
    A plugin to add plain text.
    """

    allow_children = False
    cache = True
    disable_child_plugins = True
    fieldsets = ((None, {"fields": ["body"]}),)
    model = PlainText
    module = PLUGINS_GROUP
    name = _("Plain text")
    render_template = "richie/plain_text/plain_text.html"

    def get_form(self, request, obj=None, change=False, **kwargs):
        """
        Add a max length validator based on what is configured in settings for the
        current placeholder.
        """
        form = super().get_form(request, obj=obj, change=change, **kwargs)

        placeholder_id = request.GET.get("placeholder_id")
        if not placeholder_id and not obj:
            return form

        if placeholder_id:
            placeholder = Placeholder.objects.only("slot").get(id=placeholder_id)
        else:
            placeholder = obj.placeholder

        max_length = getattr(settings, "RICHIE_PLAINTEXT_MAXLENGTH", {}).get(
            placeholder.slot
        )
        if max_length:
            form.base_fields["body"].validators.append(MaxLengthValidator(max_length))

        return form

    def render(self, context, instance, placeholder):
        """
        Build plugin context passed to its template to perform rendering
        """
        context = super().render(context, instance, placeholder)
        context["body"] = instance.body
        return context



"""Parameters that define how the demo site will be built."""
from django.conf import settings

from .utils import pick_image

DEFAULT_DEMO_SITE_DOMAIN = "localhost:8070"
DEFAULT_LMS_ENDPOINT = "http://localhost:8073"

NB_OBJECTS = {
    "courses": 75,
    "course_courseruns": 5,
    "course_icons": 3,
    "course_organizations": 3,
    "course_persons": 6,
    "course_subjects": 2,
    "person_organizations": 3,
    "person_subjects": 3,
    "organizations": 24,
    "licences": 5,
    "persons": 50,
    "blogposts": 26,
    "blogpost_levels": 1,
    "blogpost_tags": 1,
    "programs": 6,
    "programs_courses": 4,
    "home_blogposts": 5,
    "home_courses": 7,
    "home_organizations": 4,
    "home_subjects": 6,
    "home_persons": 4,
    "home_programs": 4,
}
NB_OBJECTS.update(getattr(settings, "RICHIE_DEMO_NB_OBJECTS", {}))

PAGES_INFO = {
    "home": {
        "title": {"en": "Home", "fr": "Accueil"},
        "in_navigation": False,
        "is_homepage": True,
        "template": "richie/homepage.html",
    },
    "blogposts": {
        "title": {"en": "News", "fr": "Actualités"},
        "in_navigation": True,
        "template": "courses/cms/blogpost_list.html",
    },
    "courses": {
        "title": {"en": "Courses", "fr": "Cours"},
        "in_navigation": True,
        "template": "search/search.html",
    },
    "categories": {
        "title": {"en": "Categories", "fr": "Catégories"},
        "in_navigation": True,
        "template": "courses/cms/category_list.html",
    },
    "organizations": {
        "title": {"en": "Organizations", "fr": "Établissements"},
        "in_navigation": True,
        "template": "courses/cms/organization_list.html",
    },
    "persons": {
        "title": {"en": "Persons", "fr": "Personnes"},
        "in_navigation": True,
        "template": "courses/cms/person_list.html",
    },
    "programs": {
        "title": {"en": "Programs", "fr": "Parcours"},
        "in_navigation": True,
        "template": "courses/cms/program_list.html",
    },
    "annex": {
        "title": {"en": "Annex", "fr": "Annexe"},
        "in_navigation": False,
        "template": "richie/single_column.html",
        "children": {
            "annex__about": {
                "title": {"en": "About", "fr": "A propos"},
                "in_navigation": True,
                "template": "richie/single_column.html",
            },
            "annex__sitemap": {
                "title": {"en": "Sitemap", "fr": "Plan de site"},
                "in_navigation": True,
                "template": "richie/single_column.html",
            },
            "login-error": {
                "title": {"en": "Login error", "fr": "Erreur de connexion"},
                "in_navigation": False,
                "template": "richie/single_column.html",
            },
        },
    },
}
PAGES_INFO.update(getattr(settings, "RICHIE_DEMO_PAGES_INFO", {}))


LEVELS_INFO = {
    "page_title": {"en": "Level", "fr": "Niveau"},
    "children": [
        {"page_title": {"en": "Beginner", "fr": "Débutant"}},
        {"page_title": {"en": "Advanced", "fr": "Avancé"}},
        {"page_title": {"en": "Expert", "fr": "Expert"}},
    ],
    "page_reverse_id": "levels",
}
LEVELS_INFO.update(getattr(settings, "RICHIE_DEMO_LEVELS_INFO", {}))

PARTNERSHIPS_INFO = {
    "page_title": {"en": "Partnership", "fr": "Partenariat"},
    "children": [
        {"page_title": {"en": "Gold", "fr": "Or"}},
        {"page_title": {"en": "Silver", "fr": "Argent"}},
        {"page_title": {"en": "Bronze", "fr": "Bronze"}},
    ],
    "page_reverse_id": "partnerships",
}
PARTNERSHIPS_INFO.update(getattr(settings, "RICHIE_DEMO_PARTNERSHIPS_INFO", {}))

TAGS_INFO = {
    "page_title": {"en": "Tag", "fr": "Etiquette"},
    "children": [
        {"page_title": {"en": "Event", "fr": "Evénement"}},
        {"page_title": {"en": "Feature", "fr": "Fonctionnalité"}},
        {"page_title": {"en": "Around the course", "fr": "Autour du cours"}},
        {"page_title": {"en": "Partner", "fr": "Partenaire"}},
        {"page_title": {"en": "Portrait", "fr": "Portrait"}},
        {"page_title": {"en": "Recruitment", "fr": "Recrutement"}},
    ],
    "page_reverse_id": "tags",
}
TAGS_INFO.update(getattr(settings, "RICHIE_DEMO_TAGS_INFO", {}))

SUBJECTS_INFO = {
    "page_title": {"en": "Subject", "fr": "Sujet"},
    "children": [
        {
            "page_title": {"en": "Science", "fr": "Sciences"},
            "children": [
                {
                    "page_title": {
                        "en": "Agronomy and Agriculture",
                        "fr": "Agronomie et Agriculture",
                    }
                },
                {"page_title": {"en": "Chemistry", "fr": "Chimie"}},
                {
                    "page_title": {
                        "en": "Discovery of the Universe",
                        "fr": "Découverte de l'Univers",
                    }
                },
                {"page_title": {"en": "Environment", "fr": "Environnement"}},
                {
                    "page_title": {
                        "en": "Mathematics and Statistics",
                        "fr": "Mathématiques et Statistiques",
                    }
                },
                {
                    "page_title": {
                        "en": "Tools for Research",
                        "fr": "Outils pour la Recherche",
                    }
                },
                {"page_title": {"en": "Physics", "fr": "Physique"}},
                {
                    "page_title": {
                        "en": "Cognitive science",
                        "fr": "Sciences cognitives",
                    }
                },
                {
                    "page_title": {
                        "en": "Earth science and science of the Universe",
                        "fr": "Sciences de la Terre et de l'Univers",
                    }
                },
                {"page_title": {"en": "Life science", "fr": "Sciences de la vie"}},
                {
                    "page_title": {
                        "en": "Engineering science",
                        "fr": "Sciences pour l'ingénieur",
                    }
                },
            ],
        },
        {
            "page_title": {
                "en": "Human and social sciences",
                "fr": "Sciences humaines et social",
            },
            "children": [
                {"page_title": {"en": "Communication", "fr": "Communication"}},
                {
                    "page_title": {
                        "en": "Creation, Arts and Design",
                        "fr": "Création, Arts et Design",
                    }
                },
                {
                    "page_title": {
                        "en": "Culture and Civilization",
                        "fr": "Cultures et Civilisations",
                    }
                },
                {
                    "page_title": {
                        "en": "Social Issues and Social Policy",
                        "fr": "Enjeux de société",
                    }
                },
                {"page_title": {"en": "Geography", "fr": "Géographie"}},
                {"page_title": {"en": "History", "fr": "Histoire"}},
                {"page_title": {"en": "Innovation", "fr": "Innovation"}},
                {"page_title": {"en": "Literature", "fr": "Lettres"}},
                {"page_title": {"en": "Media", "fr": "Médias"}},
                {"page_title": {"en": "Philosophy", "fr": "Philosophie"}},
                {
                    "page_title": {
                        "en": "Political science",
                        "fr": "Sciences politiques",
                    }
                },
                {
                    "page_title": {
                        "en": "International relations",
                        "fr": "Relations internationales",
                    }
                },
                {"page_title": {"en": "Sports", "fr": "Sport"}},
            ],
        },
        {"page_title": {"en": "Law", "fr": "Droit et juridique"}},
        {"page_title": {"en": "Economy and Finance", "fr": "Economie et Finance"}},
        {
            "page_title": {
                "en": "Education and Training",
                "fr": "Education et formation",
            }
        },
        {"page_title": {"en": "Management", "fr": "Management"}},
        {"page_title": {"en": "Entrepreneurship", "fr": "Entreprenariat"}},
        {
            "page_title": {"en": "Computer science", "fr": "Informatique"},
            "children": [
                {
                    "page_title": {
                        "en": "Digital and Technology",
                        "fr": "Numérique et Technologie",
                    }
                },
                {
                    "page_title": {
                        "en": "Telecommunication and Networks",
                        "fr": "Télécommunications et Réseaux",
                    }
                },
                {"page_title": {"en": "Coding", "fr": "Programmation"}},
            ],
        },
        {"page_title": {"en": "Languages", "fr": "Langues"}},
        {"page_title": {"en": "Education and career guidance", "fr": "Orientation"}},
        {"page_title": {"en": "Health", "fr": "Santé"}},
    ],
    "page_reverse_id": "subjects",
}
SUBJECTS_INFO.update(getattr(settings, "RICHIE_DEMO_SUBJECTS_INFO", {}))

ICONS_INFO = {
    "page_title": {"en": "Icons", "fr": "Icônes"},
    "children": [
        {
            "page_title": {"en": "Academic", "fr": "Diplomant"},
            "color": "#005c08",
            "fill_icon": pick_image("icons")("academic.png"),
        },
        {
            "page_title": {"en": "Accessible", "fr": "Accessible"},
            "color": "#00a1d6",
            "fill_icon": pick_image("icons")("accessible.png"),
        },
        {
            "page_title": {"en": "Closed caption", "fr": "Malentendants"},
            "color": "#a11000",
            "fill_icon": pick_image("icons")("cc.png"),
        },
        {
            "page_title": {"en": "Certificate", "fr": "Certifiant"},
            "color": "#ffc400",
            "fill_icon": pick_image("icons")("certificate.png"),
        },
        {
            "page_title": {"en": "Subtitles", "fr": "Sous-titres"},
            "color": "#6d00ba",
            "fill_icon": pick_image("icons")("subtitles.png"),
        },
    ],
    "page_reverse_id": "icons",
}
ICONS_INFO.update(getattr(settings, "RICHIE_DEMO_ICONS_INFO", {}))

HOMEPAGE_CONTENT = {
    "en": {
        "banner_title": "Welcome to Richie",
        "banner_content": (
            """<h1 class="hero-intro__title">Welcome <strong>to Richie</strong></h1>"""
            """<p>It works! This is the default homepage for the Richie CMS.</p>"""
        ),
        "banner_template": "richie/large_banner/hero-intro.html",
        "button_template_name": "button-caesura",
        "section_template": "richie/section/section.html",
        "blogposts_title": "Last news",
        "blogposts_button_title": "More news",
        "courses_title": "Popular courses",
        "courses_button_title": "More courses",
        "organizations_title": "Universities",
        "organizations_button_title": "More universities",
        "persons_title": "Persons",
        "persons_button_title": "More persons",
        "programs_title": "Programs",
        "programs_button_title": "More programs",
        "subjects_title": "Subjects",
        "subjects_button_title": "More subjects",
    },
    "fr": {
        "banner_title": "Bienvenue sur Richie",
        "banner_content": (
            """<h1 class="hero-intro__title">Bienvenue <strong>sur Richie</strong></h1>"""
            """<p>Ça marche ! Ceci est la page d'accueil par défaut du CMS Richie.</p>"""
        ),
        "banner_template": "richie/large_banner/hero-intro.html",
        "button_template_name": "button-caesura",
        "section_template": "richie/section/section.html",
        "blogposts_title": "Actualités récentes",
        "blogposts_button_title": "Plus d'actualités",
        "courses_title": "Cours à la une",
        "courses_button_title": "Plus de cours",
        "organizations_title": "Universités",
        "organizations_button_title": "Plus d'universités",
        "subjects_title": "Thématiques",
        "subjects_button_title": "Plus de thématiques",
        "persons_title": "Personnes",
        "persons_button_title": "Plus de personnes",
        "programs_title": "Parcours",
        "programs_button_title": "Plus de parcours",
    },
}
HOMEPAGE_CONTENT.update(getattr(settings, "RICHIE_DEMO_HOMEPAGE_CONTENT", {}))

SINGLECOLUMN_CONTENT = {
    "en": {
        "banner_title": "Single column template sample",
        "banner_content": "It works! This is a single column page.",
        "banner_template": "richie/large_banner/hero-intro.html",
        "button_template_name": "button-caesura",
        "section_sample_title": "A sample section",
        "section_sample_button_title": "More!",
        "section_sample_template": "richie/section/section.html",
    },
    "fr": {
        "banner_title": "Exemple de template avec une colonne unique",
        "banner_content": "Ça marche ! Ceci est une page d'une colonne.",
        "banner_template": "richie/large_banner/hero-intro.html",
        "button_template_name": "button-caesura",
        "section_sample_title": "Une section d'exemple",
        "section_sample_button_title": "Plus !",
        "section_sample_template": "richie/section/section.html",
    },
}
SINGLECOLUMN_CONTENT.update(getattr(settings, "RICHIE_DEMO_SINGLECOLUMN_CONTENT", {}))

FOOTER_CONTENT = {
    "en": [
        {"name": "About", "internal_link": "annex__about"},
        {"name": "Sitemap", "internal_link": "annex__sitemap"},
        {"name": "Style guide", "external_link": "/styleguide/"},
        {
            "title": "Richie community",
            "items": [
                {"name": "Website", "external_link": "https://richie.education"},
                {
                    "name": "Github",
                    "external_link": "https://github.com/openfun/richie",
                },
                {
                    "name": "Site factory",
                    "external_link": "https://github.com/openfun/richie-site-factory",
                },
                {
                    "name": "Example site",
                    "external_link": "https://www.fun-campus.fr",
                },
            ],
        },
    ],
    "fr": [
        {"name": "A propos", "internal_link": "annex__about"},
        {"name": "Plan du site", "internal_link": "annex__sitemap"},
        {"name": "Style guide", "external_link": "/styleguide/"},
        {
            "title": "Communauté Richie",
            "items": [
                {"name": "Site web", "external_link": "https://richie.education"},
                {
                    "name": "Github",
                    "external_link": "https://github.com/openfun/richie",
                },
                {
                    "name": "Usine à sites",
                    "external_link": "https://github.com/openfun/richie-site-factory",
                },
                {
                    "name": "Site exemple",
                    "external_link": "https://www.fun-campus.fr",
                },
            ],
        },
    ],
}
FOOTER_CONTENT.update(getattr(settings, "RICHIE_DEMO_FOOTER_CONTENT", {}))

COURSE_CONTENT = {
    "en": {"partners_title": "Partners", "sponsors_title": "Sponsors"},
    "fr": {"partners_title": "Partenaires", "sponsors_title": "Sponsors"},
}
COURSE_CONTENT.update(getattr(settings, "RICHIE_DEMO_COURSE_CONTENT", {}))

SITEMAP_PAGE_PARAMS = {
    "blogposts": {"max_depth": 1},
    "courses": {"max_depth": 1},
    "categories": {},
    "organizations": {"max_depth": 1},
    "persons": {"max_depth": 1},
    "programs": {"max_depth": 1},
    "annex": {"include_root_page": False},
}



# Generated by Django 2.2.11 on 2020-03-09 22:43

from django.db import migrations, models

import richie.apps.core.fields.multiselect


class Migration(migrations.Migration):

    dependencies = [
        ("courses", "0013_migrate_data_translated_licence_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="blogpostpluginmodel",
            name="variant",
            field=models.CharField(
                blank=True,
                choices=[(None, "Default")],
                help_text="Optional glimpse variant for a custom look.",
                max_length=50,
                null=True,
                verbose_name="variant",
            ),
        ),
    ]
